# Copyright (c) 2012 Santosh Philip
# Copyright (c) 2021  Dimitris Mantas
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""use epbunch"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from itertools import chain

from eppy.EPlusInterfaceFunctions import readidf
import eppy.bunchhelpers as bunchhelpers
from eppy.EPlusInterfaceFunctions.structures import CaseInsensitiveDict
from eppy.bunch_subclass import EpBunch

# from eppy.bunch_subclass import fieldnames, fieldvalues
import eppy.iddgaps as iddgaps
import eppy.function_helpers as fh
from eppy.idf_msequence import Idf_MSequence


class NoIDDFieldsError(Exception):
    pass


def iddversiontuple(afile):
    """given the idd file or filehandle, return the version handle"""

    def versiontuple(vers):
        """version tuple"""
        return tuple([int(num) for num in vers.split(".")])

    try:
        fhandle = open(afile, "rb")
    except TypeError:
        fhandle = afile
    line1 = fhandle.readline()
    try:
        line1 = line1.decode("ISO-8859-2")
    except AttributeError:
        pass
    line = line1.strip()
    if line1 == "":
        return (0,)
    vers = line.split()[-1]
    return versiontuple(vers)


def makeabunch(commdct, obj, obj_i, debugidd=True, block=None):
    """make a bunch from the object"""
    objidd = commdct[obj_i]
    objfields = [comm.get("field") for comm in commdct[obj_i]]
    objfields[0] = ["key"]
    objfields = [field[0] for field in objfields]
    obj_fields = [bunchhelpers.makefieldname(field) for field in objfields]
    bobj = EpBunch(obj, obj_fields, objidd)
    # TODO : test for len(obj) > len(obj_fields)
    # that will be missing fields in idd file
    # do we throw an exception here ????? YES !!!!!
    if debugidd:
        if len(obj) > len(obj_fields):
            n = len(obj) - len(obj_fields)
            extlst = extension_of_extensible(commdct[obj_i], block[obj_i], n)
            errortext = "idfobject with key '{}' & first field '{}' has {} fields while the idd for '{}' has only {} fields. Add the following fields to the object '{}' in file Energy+.idd '{};'".format(
                obj[0].upper(),
                obj[1],
                len(obj) - 1,
                obj[0].upper(),
                len(obj_fields) - 1,
                obj[0].upper(),
                ", ".join(extlst),
            )
            # "idfobject with key 'TIMESTEP' & first field '44' has 2 fields while the idd for 'TIMESTEP' has only 1 fields. Add the following fields to object 'TIMESTEP' in file Energy+.idd A5, A6;'"
            # print(block[obj_i])
            # print(errortext)
            print(extlst)
            raise NoIDDFieldsError(errortext)
    return bobj


def makebunches(data, commdct):
    """make bunches with data"""
    bunchdt = CaseInsensitiveDict()
    ddtt, dtls = data.dt, data.dtls
    for obj_i, key in enumerate(dtls):
        key = key.upper()
        bunchdt[key] = []
        objs = ddtt[key]
        for obj in objs:
            bobj = makeabunch(commdct, obj, obj_i)
            bunchdt[key].append(bobj)
    return bunchdt


def makebunches_alter(data, commdct, theidf, block=None):
    """make bunches with data"""
    bunchdt = CaseInsensitiveDict()
    dt, dtls = data.dt, data.dtls
    for obj_i, key in enumerate(dtls):
        key = key.upper()
        objs = dt[key]
        list1 = []
        for obj in objs:
            bobj = makeabunch(commdct, obj, obj_i, block=block)
            list1.append(bobj)
        bunchdt[key] = Idf_MSequence(list1, objs, theidf)
    return bunchdt


class ConvInIDD(object):
    """hold the conversion function to integer, real and no_type"""

    def no_type(self, x, avar):
        if avar.startswith("N"):  # is a number if it starts with N
            try:
                return float(x)  # in case x=autosize
            except ValueError as e:
                return x
        else:
            return x  # starts with A, is not a number

    def integer(self, x, y):
        try:
            return int(x)
        except ValueError as e:
            return x

    def real(self, x, y):
        try:
            return float(x)
        except ValueError as e:
            return x

    def conv_dict(self):
        """dictionary of conversion"""
        return dict(integer=self.integer, real=self.real, no_type=self.no_type)


# remove this one
def convertfields_old(key_comm, obj, inblock=None):
    """convert the float and interger fields"""
    convinidd = ConvInIDD()
    typefunc = dict(integer=convinidd.integer, real=convinidd.real)
    types = []
    for comm in key_comm:
        types.append(comm.get("type", [None])[0])
    convs = [typefunc.get(typ, convinidd.no_type) for typ in types]
    try:
        inblock = list(inblock)
    except TypeError as e:
        inblock = ["does not start with N"] * len(obj)
    for i, (val, conv, avar) in enumerate(zip(obj, convs, inblock)):
        if i == 0:
            # inblock[0] is the key
            pass
        else:
            val = conv(val, inblock[i])
        obj[i] = val
    return obj


def convertafield(field_comm, field_val, field_iddname):
    """convert field based on field info in IDD"""
    convinidd = ConvInIDD()
    field_typ = field_comm.get("type", [None])[0]
    conv = convinidd.conv_dict().get(field_typ, convinidd.no_type)
    return conv(field_val, field_iddname)


def convertfields(key_comm, obj, inblock=None):
    """convert based on float, integer, and A1, N1"""
    # f_ stands for field_
    convinidd = ConvInIDD()
    if not inblock:
        inblock = ["does not start with N"] * len(obj)
    for i, (f_comm, f_val, f_iddname) in enumerate(zip(key_comm, obj, inblock)):
        if i == 0:
            # inblock[0] is the iddobject key. No conversion here
            pass
        else:
            obj[i] = convertafield(f_comm, f_val, f_iddname)
    return obj


def convertallfields(data, commdct, block=None):
    """docstring for convertallfields"""
    # import pdbdb; pdb.set_trace()
    for key in list(data.dt.keys()):
        objs = data.dt[key]
        for i, obj in enumerate(objs):
            key_i = data.dtls.index(key)
            key_comm = commdct[key_i]
            try:
                inblock = block[key_i]
            except TypeError as e:
                inblock = None
            obj = convertfields(key_comm, obj, inblock)
            objs[i] = obj


def addfunctions(dtls, bunchdt):
    """add functions to the objects"""
    snames = [
        "BuildingSurface:Detailed",
        "Wall:Detailed",
        "RoofCeiling:Detailed",
        "Floor:Detailed",
        "FenestrationSurface:Detailed",
        "Shading:Site:Detailed",
        "Shading:Building:Detailed",
        "Shading:Zone:Detailed",
    ]
    for sname in snames:
        if sname.upper() in bunchdt:
            surfaces = bunchdt[sname.upper()]
            for surface in surfaces:
                func_dict = {
                    "area": fh.area,
                    "height": fh.height,  # not working correctly
                    "width": fh.width,  # not working correctly
                    "azimuth": fh.azimuth,
                    "tilt": fh.tilt,
                    "coords": fh.getcoords,  # needed for debugging
                }
                try:
                    surface.__functions.update(func_dict)
                except KeyError as e:
                    surface.__functions = func_dict
    # add common functions
    # for name in dtls:
    #     for idfobject in bunchdt[name]:
    # idfobject.__functions
    #         idfobject['__functions']['fieldnames'] = fieldnames
    #         idfobject['__functions']['fieldvalues'] = fieldvalues
    #         idfobject['__functions']['getrange'] = GetRange(idfobject)
    #         idfobject['__functions']['checkrange'] = CheckRange(idfobject)


def addfunctions2new(abunch, key):
    """add functions to a new bunch/munch object"""
    snames = [
        "BuildingSurface:Detailed",
        "Wall:Detailed",
        "RoofCeiling:Detailed",
        "Floor:Detailed",
        "FenestrationSurface:Detailed",
        "Shading:Site:Detailed",
        "Shading:Building:Detailed",
        "Shading:Zone:Detailed",
    ]
    snames = [sname.upper() for sname in snames]
    if key in snames:
        func_dict = {
            "area": fh.area,
            "height": fh.height,  # not working correctly
            "width": fh.width,  # not working correctly
            "azimuth": fh.azimuth,
            "tilt": fh.tilt,
            "coords": fh.getcoords,  # needed for debugging
        }
        try:
            abunch.__functions.update(func_dict)
        except KeyError as e:
            abunch.__functions = func_dict
    return abunch


def idfreader(fname, iddfile, conv=True):
    """read idf file and return bunches"""
    data, commdct, idd_index = readidf.readdatacommdct(fname, iddfile=iddfile)
    if conv:
        convertallfields(data, commdct)
    # fill gaps in idd
    ddtt, dtls = data.dt, data.dtls
    # skiplist = ["TABLE:MULTIVARIABLELOOKUP"]
    nofirstfields = iddgaps.missingkeys_standard(
        commdct, dtls, skiplist=["TABLE:MULTIVARIABLELOOKUP"]
    )
    iddgaps.missingkeys_nonstandard(None, commdct, dtls, nofirstfields)
    bunchdt = makebunches(data, commdct)
    return bunchdt, data, commdct, idd_index


def idfreader1(fname, iddfile, theidf, conv=True, commdct=None, block=None):
    """read idf file and return bunches"""
    versiontuple = iddversiontuple(iddfile)
    # import pdb; pdb.set_trace()
    block, data, commdct, idd_index = readidf.readdatacommdct1(
        fname, iddfile=iddfile, commdct=commdct, block=block
    )
    if conv:
        convertallfields(data, commdct, block)
    # fill gaps in idd
    ddtt, dtls = data.dt, data.dtls
    if versiontuple < (8,):
        skiplist = ["TABLE:MULTIVARIABLELOOKUP"]
    else:
        skiplist = None
    nofirstfields = iddgaps.missingkeys_standard(commdct, dtls, skiplist=skiplist)
    iddgaps.missingkeys_nonstandard(block, commdct, dtls, nofirstfields)
    # bunchdt = makebunches(data, commdct)
    bunchdt = makebunches_alter(data, commdct, theidf, block)
    return bunchdt, block, data, commdct, idd_index, versiontuple


def getextensible(objidd):
    """return the extensible from the idd"""
    keys = objidd[0].keys()
    extkey = [key for key in keys if key.startswith("extensible")]
    if extkey:
        extens = extkey[0].split(":")[-1]
        return int(extens)
    else:
        return None


def endof_extensible(extensible, thisblock):
    """get the vars from where extension happens"""
    return thisblock[-extensible:]


def extension_of_extensible(objidd, objblock, n):
    """generate the list of new vars needed to extend by n"""
    ext = getextensible(objidd)
    print(n, type(n), ext, type(ext))
    # n = int(n / ext)
    n = n // ext
    print(n, type(n))
    lastvars = endof_extensible(ext, objblock)
    print("ext, lastvars", ext, lastvars)
    alpha_lastvars = [i[0] for i in lastvars]
    int_lastvars = [int(i[1:]) for i in lastvars]
    lst = []
    for alpha, start in zip(alpha_lastvars, int_lastvars):
        step = alpha_lastvars.count(alpha)
        rng = range(start + step, start + 1 + n * step, step)
        lst.append(["{}{}".format(alpha, item) for item in rng])

    from itertools import chain

    return list(chain(*zip(*lst)))


# complete -- remove this junk below
# working code - working on it now.
# N3, A4, M8, A5
#
# N3, A4, M8, A5
# N4, A6, M9, A7
# N5, A8, M10, A9
# N6, A10, M11, A11


# ref = idf1.newidfobject("Refrigeration:WalkIn".upper())
# lastvars = ["N3", "A4", "M8", "A5"]
# lastvars = [u'A18',
#  u'N29',
#  u'N30',
#  u'N31',
#  u'N32',
#  u'N33',
#  u'A19',
#  u'N34',
#  u'N35',
#  u'N36',
#  u'A20',
#  u'A21']
# alpha_lastvars = [i[0] for i in lastvars]
# int_lastvars = [int(i[1:]) for i in lastvars]
#
#
#
# n = 2
#
# lst = []
# for alpha, start in zip(alpha_lastvars, int_lastvars):
#     step = alpha_lastvars.count(alpha)
#     rng = range(start +1, start + 1 + n * step, step)
#     lst.append(["{}{}".format(alpha, item) for item in rng])
#
# from itertools import chain
# c = list(chain(*zip(*lst)))
#
#



# Copyright (c) 2020 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
# -*- coding: utf-8 -*-
"""py.test for fasthtml"""


from io import StringIO
from pathlib import Path
import pytest
from eppy.results import readhtml
from eppy.results import fasthtml


class SampleHtml(object):
    """Sample html text"""

    def __init__(self):
        self.html = """<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN""http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title> California DGS- 10th & O Street Offices 2 SACRAMENTO-EXECUTIVE_724830_CZ2010 ** SACRAMENTO-EXECUTIVE-AP - USA Custom-724830 WMO#=724830
  2019-08-19
  10:51:57
 - EnergyPlus</title>
</head>
<body>
<p><a href="#toc" style="float: right">Table of Contents</a></p>
<a name=top></a>
<p>Program Version:<b>EnergyPlus, Version 8.5.0-c87e61b44b, YMD=2019.08.19 10:39</b></p>
<p>Tabular Output Report in Format: <b>HTML</b></p>
<p>Building: <b>California DGS- 10th & O Street Offices 2</b></p>
<p>Environment: <b>SACRAMENTO-EXECUTIVE_724830_CZ2010 ** SACRAMENTO-EXECUTIVE-AP - USA Custom-724830 WMO#=724830</b></p>
<p>Simulation Timestamp: <b>2019-08-19
  10:51:57</b></p>
<hr>
<p><a href="#toc" style="float: right">Table of Contents</a></p>
<a name=AnnualBuildingUtilityPerformanceSummary::EntireFacility></a>
<p>Report:<b> Annual Building Utility Performance Summary</b></p>
<p>For:<b> Entire Facility</b></p>
<p>Timestamp: <b>2019-08-19
    10:51:57</b></p>
<b>Values gathered over      8760.00 hours</b><br><br>
<b></b><br><br>
<b>Site and Source Energy</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Site and Source Energy-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Total Energy [kBtu]</td>
    <td align="right">Energy Per Total Building Area [kBtu/ft2]</td>
    <td align="right">Energy Per Conditioned Building Area [kBtu/ft2]</td>
  </tr>
  <tr>
    <td align="right">Total Site Energy</td>
    <td align="right"> 12372300.67</td>
    <td align="right">       27.07</td>
    <td align="right">       28.11</td>
  </tr>
  <tr>
    <td align="right">Net Site Energy</td>
    <td align="right"> 12372300.67</td>
    <td align="right">       27.07</td>
    <td align="right">       28.11</td>
  </tr>
  <tr>
    <td align="right">Total Source Energy</td>
    <td align="right"> 31924186.59</td>
    <td align="right">       69.85</td>
    <td align="right">       72.53</td>
  </tr>
  <tr>
    <td align="right">Net Source Energy</td>
    <td align="right"> 31924186.59</td>
    <td align="right">       69.85</td>
    <td align="right">       72.53</td>
  </tr>
</table>
<br><br>
<b>Site to Source Energy Conversion Factors</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Site to Source Energy Conversion Factors-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Site=>Source Conversion Factor</td>
  </tr>
  <tr>
    <td align="right">Electricity</td>
    <td align="right">       3.167</td>
  </tr>
  <tr>
    <td align="right">Natural Gas</td>
    <td align="right">       1.084</td>
  </tr>
  <tr>
    <td align="right">District Cooling</td>
    <td align="right">       1.056</td>
  </tr>
  <tr>
    <td align="right">District Heating</td>
    <td align="right">       3.613</td>
  </tr>
  <tr>
    <td align="right">Steam</td>
    <td align="right">       0.300</td>
  </tr>
  <tr>
    <td align="right">Gasoline</td>
    <td align="right">       1.050</td>
  </tr>
  <tr>
    <td align="right">Diesel</td>
    <td align="right">       1.050</td>
  </tr>
  <tr>
    <td align="right">Coal</td>
    <td align="right">       1.050</td>
  </tr>
  <tr>
    <td align="right">Fuel Oil #1</td>
    <td align="right">       1.050</td>
  </tr>
  <tr>
    <td align="right">Fuel Oil #2</td>
    <td align="right">       1.050</td>
  </tr>
  <tr>
    <td align="right">Propane</td>
    <td align="right">       1.050</td>
  </tr>
  <tr>
    <td align="right">Other Fuel 1</td>
    <td align="right">       1.000</td>
  </tr>
  <tr>
    <td align="right">Other Fuel 2</td>
    <td align="right">       1.000</td>
  </tr>
</table>
<br><br>
<b>Building Area</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Building Area-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Area [ft2]</td>
  </tr>
  <tr>
    <td align="right">Total Building Area</td>
    <td align="right">   457021.76</td>
  </tr>
  <tr>
    <td align="right">Net Conditioned Building Area</td>
    <td align="right">   440178.05</td>
  </tr>
  <tr>
    <td align="right">Unconditioned Building Area</td>
    <td align="right">    16843.71</td>
  </tr>
</table>
<br><br>
<b>End Uses</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_End Uses-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Electricity [kBtu]</td>
    <td align="right">Natural Gas [kBtu]</td>
    <td align="right">Additional Fuel [kBtu]</td>
    <td align="right">District Cooling [kBtu]</td>
    <td align="right">District Heating [kBtu]</td>
    <td align="right">Water [gal]</td>
  </tr>
  <tr>
    <td align="right">Heating</td>
    <td align="right">      699.76</td>
    <td align="right">  3484218.01</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Cooling</td>
    <td align="right">   778887.08</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Interior Lighting</td>
    <td align="right">  1451508.91</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Exterior Lighting</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Interior Equipment</td>
    <td align="right">  5243267.78</td>
    <td align="right">      606.58</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Exterior Equipment</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Fans</td>
    <td align="right">  1011588.31</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Pumps</td>
    <td align="right">   308846.84</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Heat Rejection</td>
    <td align="right">    92677.39</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">  1370313.32</td>
  </tr>
  <tr>
    <td align="right">Humidification</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Heat Recovery</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Water Systems</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Refrigeration</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Generators</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">Total End Uses</td>
    <td align="right">  8887476.08</td>
    <td align="right">  3484824.60</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">  1370313.32</td>
  </tr>
</table>
<i>Note: Natural gas appears to be the principal heating source based on energy usage.</i>
<br><br>
<b>End Uses By Subcategory</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_End Uses By Subcategory-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Subcategory</td>
    <td align="right">Electricity [kBtu]</td>
    <td align="right">Natural Gas [kBtu]</td>
    <td align="right">Additional Fuel [kBtu]</td>
    <td align="right">District Cooling [kBtu]</td>
    <td align="right">District Heating [kBtu]</td>
    <td align="right">Water [gal]</td>
  </tr>
  <tr>
    <td align="right">Heating</td>
    <td align="right">Boiler</td>
    <td align="right">        0.00</td>
    <td align="right">  3484218.01</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">&nbsp;</td>
    <td align="right">Boiler Parasitic</td>
    <td align="right">      699.76</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Cooling</td>
    <td align="right">General</td>
    <td align="right">   778887.08</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Interior Lighting</td>
    <td align="right">ComplianceLtg</td>
    <td align="right">  1355763.53</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">&nbsp;</td>
    <td align="right">NonComplianceLtg</td>
    <td align="right">    95745.38</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Exterior Lighting</td>
    <td align="right">General</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Interior Equipment</td>
    <td align="right">Receptacle</td>
    <td align="right">  4832413.78</td>
    <td align="right">      606.58</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">&nbsp;</td>
    <td align="right">Process</td>
    <td align="right">    59134.94</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">&nbsp;</td>
    <td align="right">Internal Transport</td>
    <td align="right">   351719.06</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Exterior Equipment</td>
    <td align="right">General</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Fans</td>
    <td align="right">General</td>
    <td align="right">   955401.72</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">&nbsp;</td>
    <td align="right">ProcessMotors</td>
    <td align="right">    56186.59</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Pumps</td>
    <td align="right">General</td>
    <td align="right">   308846.84</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Heat Rejection</td>
    <td align="right">General</td>
    <td align="right">    92677.39</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">  1370313.32</td>
  </tr>
  <tr>
    <td align="right">Humidification</td>
    <td align="right">General</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Heat Recovery</td>
    <td align="right">General</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Water Systems</td>
    <td align="right">General</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Refrigeration</td>
    <td align="right">General</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Generators</td>
    <td align="right">General</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
</table>
<br><br>
<b>Normalized Metrics</b><br><br>
<b>Utility Use Per Conditioned Floor Area</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Utility Use Per Conditioned Floor Area-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Electricity Intensity [kBtu/ft2]</td>
    <td align="right">Natural Gas Intensity [kBtu/ft2]</td>
    <td align="right">Additional Fuel Intensity [kBtu/ft2]</td>
    <td align="right">District Cooling Intensity [kBtu/ft2]</td>
    <td align="right">District Heating Intensity [kBtu/ft2]</td>
    <td align="right">Water Intensity [gal/ft2]</td>
  </tr>
  <tr>
    <td align="right">Lighting</td>
    <td align="right">        3.30</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">HVAC</td>
    <td align="right">        4.98</td>
    <td align="right">        7.92</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        3.11</td>
  </tr>
  <tr>
    <td align="right">Other</td>
    <td align="right">       11.91</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Total</td>
    <td align="right">       20.19</td>
    <td align="right">        7.92</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        3.11</td>
  </tr>
</table>
<br><br>
<b>Utility Use Per Total Floor Area</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Utility Use Per Total Floor Area-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Electricity Intensity [kBtu/ft2]</td>
    <td align="right">Natural Gas Intensity [kBtu/ft2]</td>
    <td align="right">Additional Fuel Intensity [kBtu/ft2]</td>
    <td align="right">District Cooling Intensity [kBtu/ft2]</td>
    <td align="right">District Heating Intensity [kBtu/ft2]</td>
    <td align="right">Water Intensity [gal/ft2]</td>
  </tr>
  <tr>
    <td align="right">Lighting</td>
    <td align="right">        3.18</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">HVAC</td>
    <td align="right">        4.80</td>
    <td align="right">        7.62</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        3.00</td>
  </tr>
  <tr>
    <td align="right">Other</td>
    <td align="right">       11.47</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Total</td>
    <td align="right">       19.45</td>
    <td align="right">        7.63</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
    <td align="right">        3.00</td>
  </tr>
</table>
<br><br>
<b>Electric Loads Satisfied</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Electric Loads Satisfied-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Electricity [kBtu]</td>
    <td align="right">Percent Electricity [%]</td>
  </tr>
  <tr>
    <td align="right">Fuel-Fired Power Generation</td>
    <td align="right">       0.000</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">High Temperature Geothermal*</td>
    <td align="right">       0.000</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Photovoltaic Power</td>
    <td align="right">       0.000</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Wind Power</td>
    <td align="right">       0.000</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Power Conversion</td>
    <td align="right">       0.000</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Net Decrease in On-Site Storage</td>
    <td align="right">       0.000</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Total On-Site Electric Sources</td>
    <td align="right">       0.000</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">Electricity Coming From Utility</td>
    <td align="right"> 8887476.075</td>
    <td align="right">      100.00</td>
  </tr>
  <tr>
    <td align="right">Surplus Electricity Going To Utility</td>
    <td align="right">       0.000</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Net Electricity From Utility</td>
    <td align="right"> 8887476.075</td>
    <td align="right">      100.00</td>
  </tr>
  <tr>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">Total On-Site and Utility Electric Sources</td>
    <td align="right"> 8887476.075</td>
    <td align="right">      100.00</td>
  </tr>
  <tr>
    <td align="right">Total Electricity End Uses</td>
    <td align="right"> 8887476.075</td>
    <td align="right">      100.00</td>
  </tr>
</table>
<br><br>
<b>On-Site Thermal Sources</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_On-Site Thermal Sources-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Heat [kBtu]</td>
    <td align="right">Percent Heat [%]</td>
  </tr>
  <tr>
    <td align="right">Water-Side Heat Recovery</td>
    <td align="right">        0.00</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">Air to Air Heat Recovery for Cooling</td>
    <td align="right">        0.00</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">Air to Air Heat Recovery for Heating</td>
    <td align="right">        0.00</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">High-Temperature Geothermal*</td>
    <td align="right">        0.00</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">Solar Water Thermal</td>
    <td align="right">        0.00</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">Solar Air Thermal</td>
    <td align="right">        0.00</td>
    <td align="right">&nbsp;</td>
  </tr>
  <tr>
    <td align="right">Total On-Site Thermal Sources</td>
    <td align="right">        0.00</td>
    <td align="right">&nbsp;</td>
  </tr>
</table>
<br><br>
<b>Water Source Summary</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Water Source Summary-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Water [gal]</td>
    <td align="right">Percent Water [%]</td>
  </tr>
  <tr>
    <td align="right">Rainwater Collection</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Condensate Collection</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Groundwater Well</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Total On Site Water Sources</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">-</td>
    <td align="right">-</td>
    <td align="right">-</td>
  </tr>
  <tr>
    <td align="right">Initial Storage</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Final Storage</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Change in Storage</td>
    <td align="right">        0.00</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">-</td>
    <td align="right">-</td>
    <td align="right">-</td>
  </tr>
  <tr>
    <td align="right">Water Supplied by Utility</td>
    <td align="right">  1370313.32</td>
    <td align="right">      100.00</td>
  </tr>
  <tr>
    <td align="right">-</td>
    <td align="right">-</td>
    <td align="right">-</td>
  </tr>
  <tr>
    <td align="right">Total On Site, Change in Storage, and Utility Water Sources</td>
    <td align="right">  1370313.32</td>
    <td align="right">      100.00</td>
  </tr>
  <tr>
    <td align="right">Total Water End Uses</td>
    <td align="right">  1370313.32</td>
    <td align="right">      100.00</td>
  </tr>
</table>
<br><br>
<b>Setpoint Not Met Criteria</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Setpoint Not Met Criteria-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Degrees [deltaF]</td>
  </tr>
  <tr>
    <td align="right">Tolerance for Zone Heating Setpoint Not Met Time</td>
    <td align="right">        1.01</td>
  </tr>
  <tr>
    <td align="right">Tolerance for Zone Cooling Setpoint Not Met Time</td>
    <td align="right">        1.01</td>
  </tr>
</table>
<br><br>
<b>Comfort and Setpoint Not Met Summary</b><br><br>
<!-- FullName:Annual Building Utility Performance Summary_Entire Facility_Comfort and Setpoint Not Met Summary-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">Facility [Hours]</td>
  </tr>
  <tr>
    <td align="right">Time Setpoint Not Met During Occupied Heating</td>
    <td align="right">      723.00</td>
  </tr>
  <tr>
    <td align="right">Time Setpoint Not Met During Occupied Cooling</td>
    <td align="right">        0.00</td>
  </tr>
  <tr>
    <td align="right">Time Not Comfortable Based on Simple ASHRAE 55-2004</td>
    <td align="right">     6403.00</td>
  </tr>
</table>
<br><br>
Note 1: An asterisk (*) indicates that the feature is not yet implemented.<br>
<hr>
</body>
</html>
"""


# def test_gets2sconversions():
#     """py.test fro gets2sconversions"""
#     data = (
#         (
#             SampleHtml().html,
#             {
#                 "Coal": 1.05,
#                 "Diesel": 1.05,
#                 "District Cooling": 1.056,
#                 "District Heating": 3.613,
#                 "Electricity": 3.167,
#                 "Fuel Oil #1": 1.05,
#                 "Fuel Oil #2": 1.05,
#                 "Gasoline": 1.05,
#                 "Natural Gas": 1.084,
#                 "Other Fuel 1": 1.0,
#                 "Other Fuel 2": 1.0,
#                 "Propane": 1.05,
#                 "Steam": 0.3,
#             },
#         ),  # html, expected
#     )
#     for html, expected in data:
#         htables = readhtml.titletable(StringIO(html))
#         result = fasthtml.gets2sconversions(htables)
#         assert result == expected
#
#
# def test_getthetable():
#     """py.test for getthetable"""
#     data = (
#         (
#             SampleHtml().html,
#             "Site and Source Energy",
#             [
#                 [
#                     "",
#                     "Total Energy [kBtu]",
#                     "Energy Per Total Building Area [kBtu/ft2]",
#                     "Energy Per Conditioned Building Area [kBtu/ft2]",
#                 ],
#                 ["Total Site Energy", 12372300.67, 27.07, 28.11],
#                 ["Net Site Energy", 12372300.67, 27.07, 28.11],
#                 ["Total Source Energy", 31924186.59, 69.85, 72.53],
#                 ["Net Source Energy", 31924186.59, 69.85, 72.53],
#             ],
#         ),  # html, header, expected
#     )
#     for html, header, expected in data:
#         htables = readhtml.titletable(StringIO(html))
#         result = fasthtml.getthetable(htables, header)
#         assert result == expected
#


def test_getnexttable():
    """py.test for getnexttable"""
    data = (
        (
            """<!-- FullName:Adaptive Comfort Summary_Entire Facility_Time Not Meeting the Adaptive Comfort Models during Occupied Hours-->
<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">ASHRAE55 90% Acceptability Limits [Hours]</td>
  </tr>
</table>
<br><br>
""",
            """<table border="1" cellpadding="4" cellspacing="0">
  <tr><td></td>
    <td align="right">ASHRAE55 90% Acceptability Limits [Hours]</td>
  </tr>
</table>
""",
        ),  # htmltxt, expected
    )
    for htmltxt, expected in data:
        fhandle = StringIO(htmltxt)
        result = fasthtml.getnexttable(fhandle)
        assert result == expected


@pytest.mark.parametrize(
    "htmltxt, header, expected",
    [
        (
            SampleHtml().html,
            "Site to Source Energy Conversion Factors",
            [
                "Site to Source Energy Conversion Factors",
                [
                    ["", "Site=>Source Conversion Factor"],
                    ["Electricity", 3.167],
                    ["Natural Gas", 1.084],
                    ["District Cooling", 1.056],
                    ["District Heating", 3.613],
                    ["Steam", 0.3],
                    ["Gasoline", 1.05],
                    ["Diesel", 1.05],
                    ["Coal", 1.05],
                    ["Fuel Oil #1", 1.05],
                    ["Fuel Oil #2", 1.05],
                    ["Propane", 1.05],
                    ["Other Fuel 1", 1.0],
                    ["Other Fuel 2", 1.0],
                ],
            ],
        ),  # htmltxt, header, expected
    ],
)
def test_tablebyname(tmp_path, htmltxt, header, expected):
    """py.test for tablebyname"""
    fhandle = StringIO(htmltxt)
    result = fasthtml.tablebyname(fhandle, header)
    assert result == expected
    # -
    # read the html from dis in 'rb' binary mode
    tmp_file = tmp_path / "eplus.htm"
    tmp_file.write_text(htmltxt)
    fhandle = open(tmp_file, "rb")
    result = fasthtml.tablebyname(fhandle, header)
    assert result == expected


@pytest.mark.parametrize(
    "htmltxt, expected",
    [
        (
            """line1
        line2
        <table stuff>
        more stuff
        </table>
        otherstuff""",
            """line1
        line2
        <table stuff>
        more stuff
        </table>
""",
        ),  # htmltxt, expected
    ],
)
def test_get_upto_nexttable(tmp_path, htmltxt, expected):
    """py.test for get_upto_nexttable"""
    fhandle = StringIO(htmltxt)
    result = fasthtml.get_upto_nexttable(fhandle)
    assert result == expected
    tmp_file = tmp_path / "eplus.htm"
    tmp_file.write_text(htmltxt)
    fhandle = open(tmp_file, "rb")
    result = fasthtml.get_upto_nexttable(fhandle)
    # assert result == expected
    # the above line did not work on windows because of different line endings
    # so we do
    result = result.strip()
    expected = expected.strip()
    for rline, line in zip(result.splitlines(), expected.splitlines()):
        assert rline.strip() == line.strip()  # removes line endings


@pytest.mark.parametrize(
    "htmltxt, index, expected",
    [
        (
            SampleHtml().html,
            1,
            (
                "Site to Source Energy Conversion Factors",
                [
                    ["", "Site=>Source Conversion Factor"],
                    ["Electricity", 3.167],
                    ["Natural Gas", 1.084],
                    ["District Cooling", 1.056],
                    ["District Heating", 3.613],
                    ["Steam", 0.3],
                    ["Gasoline", 1.05],
                    ["Diesel", 1.05],
                    ["Coal", 1.05],
                    ["Fuel Oil #1", 1.05],
                    ["Fuel Oil #2", 1.05],
                    ["Propane", 1.05],
                    ["Other Fuel 1", 1.0],
                    ["Other Fuel 2", 1.0],
                ],
            ),
        ),  #  htmltxt, index, expected
    ],
)
def test_tablebyindex(tmp_path, htmltxt, index, expected):
    """py.test for tablebyindex"""
    fhandle = StringIO(htmltxt)
    result = fasthtml.tablebyindex(fhandle, index)
    assert result == expected
    tmp_file = tmp_path / "eplus.htm"
    tmp_file.write_text(htmltxt)
    fhandle = open(tmp_file, "rb")
    result = fasthtml.tablebyindex(fhandle, index)
    assert result == expected



# Copyright (c) 2012, 2020 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""pytest for bunchhelpers"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import eppy.bunchhelpers as bunchhelpers


def test_onlylegalchar():
    """py.test for onlylegalchar"""
    data = (
        ("abc", "abc"),  # name, newname
        ("abc {mem}", "abc mem"),  # name, newname
        ("abc {mem} #1", "abc mem 1"),  # name, newname
    )
    for name, newname in data:
        result = bunchhelpers.onlylegalchar(name)
        assert result == newname


def test_makefieldname():
    """py.test for makefieldname"""
    data = (
        ("aname", "aname"),  # namefromidd, bunchname
        ("a name", "a_name"),  # namefromidd, bunchname
        ("a name #1", "a_name_1"),  # namefromidd, bunchname
    )
    for namefromidd, bunchname in data:
        result = bunchhelpers.makefieldname(namefromidd)
        assert result == bunchname


def testintinlist():
    """pytest for intinlist"""
    data = (
        ("this is", False),  # lst, hasint
        ("this is 1", True),  # lst, hasint
        ("this 54 is ", True),  # lst, hasint
    )
    for lst, hasint in data:
        result = bunchhelpers.intinlist(lst)
        assert result == hasint


def test_replaceint():
    """pytest for replaceint"""
    data = (
        ("this is", "this is"),  # fname, newname
        ("this is 54", "this is %s"),  # fname, newname
        # ('this is #54', 'this is %s'), # fname, newname
    )
    for fname, newname in data:
        result = bunchhelpers.replaceint(fname)
        assert result == newname


def test_scientificnotation():
    """py.test for scientificnotation"""
    data = (
        (100000, 3, "1.000000e+05"),  # val, width, expected
        (10, 3, 10),  # val, width, expected
        ("gumby", 3, "gumby"),  # val, width, expected
    )
    for val, width, expected in data:
        result = bunchhelpers.scientificnotation(val, width)
        assert result == expected



# Copyright (c) 2012 Tuan Tran
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

""" pytest for area_zone.py"""
# Wrote by Tuan Tran trantuan@hawaii.edu / tranhuuanhtuan@gmail.com
# School of Architecture, University of Hawaii at Manoa
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import eppy.geometry.area_zone as area_zone
from eppy.pytest_helpers import almostequal


def test_area():
    """test for area of a zone"""

    data = (
        ([(0, 0, 0), (1, 0, 1), (1, 1, 0), (0, 1, 1)], 1),  # polygon, answer,
        ([(0, 0, 0), (1, 0, 0), (1, 0, 1), (0, 0, 1)], 0),
        ([(0, 0, 0), (0, 1, 0), (0, 1, 1), (0, 0, 1)], 0),
        ([(0, 0, 4), (5, 0, 4), (5, 5, 6), (0, 5, 6)], 25),
    )
    for poly, answer in data:
        result = area_zone.area(poly)
        assert almostequal(answer, result, places=4) == True



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""py.test for EPlusInterfaceFunctions.parse_idd.py"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import eppy.EPlusInterfaceFunctions.parse_idd as parse_idd


def test_extractidddata():
    """py.test for extractidddata"""
    assert 1 == 1


def test_removeblanklines():
    """py.test for removeblanklines"""
    tdata = (
        # unix line endings
        ("1\n2", "1\n2"),  # astr, nstr
        ("1\n\n2", "1\n2"),  # astr, nstr
        ("1\n   \n2", "1\n2"),  # astr, nstr
        # dos line endings
        ("1\r\n2", "1\n2"),  # astr, nstr
        ("1\r\n\r\n2", "1\n2"),  # astr, nstr
        ("1\r\n   \r\n2", "1\n2"),  # astr, nstr
        # mac line endings
        ("1\r2", "1\n2"),  # astr, nstr
        ("1\r\r2", "1\n2"),  # astr, nstr
        ("1\r   \r2", "1\n2"),  # astr, nstr
        # mixed unix and dos line endings
        ("1\r\n2\n3", "1\n2\n3"),  # astr, nstr
    )
    for astr, nstr in tdata:
        result = parse_idd.removeblanklines(astr)
        # print(astr.__repr__(), nstr.__repr__(), result.__repr__())
        assert result == nstr



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""from BUILDINGSURFACE:DETAILED and FENESTRATIONSURFACE:DETAILED make a wall
floor, celiling etc or a window"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

# key fields:
# Name
# Surface Type
#     key Floor
#     key Wall
#     key Ceiling
#     key Roof
# Construction Name
# Zone Name
# Outside Boundary Condition
#     key Adiabatic
#     key Surface
#     key Zone
#     key Outdoors
#     key Ground
#     key GroundFCfactorMethod
#     key OtherSideCoefficients
#     key OtherSideConditionsModel
#     key GroundSlabPreprocessorAverage
#     key GroundSlabPreprocessorCore
#     key GroundSlabPreprocessorPerimeter
#     key GroundBasementPreprocessorAverageWall
#     key GroundBasementPreprocessorAverageFloor
#     key GroundBasementPreprocessorUpperWall
#     key GroundBasementPreprocessorLowerWall
# Outside Boundary Condition Object
#
# 'FENESTRATIONSURFACE:DETAILED',
# Surface_Type
#        key Window
#        key Door
#        key GlassDoor
#        key TubularDaylightDome
#        key TubularDaylightDiffuser
#
#
# 'BUILDINGSURFACE:DETAILED',
# (simple_surface, Surface_Type, Outside_Boundary_Condition)
# ----------------------------------------------------------
# ('WALL:EXTERIOR', Wall, Outdoors)
# ('WALL:ADIABATIC',Wall, Adiabatic)
# ('WALL:UNDERGROUND', Wall, s.startswith('Ground'))
# ('WALL:INTERZONE', Wall, Surface OR Zone)
# ('ROOF', Roof, None or Outdoor)
# ('CEILING:ADIABATIC', Ceiling, Adiabatic)
# ('CEILING:INTERZONE', Ceiling, Surface OR Zone or OtherSideCoefficients)
# ('FLOOR:GROUNDCONTACT', Floor, s.startswith('Ground'))
# ('FLOOR:ADIABATIC', Floor, Adiabatic)
# ('FLOOR:INTERZONE', Floor, Surface OR Zone or OtherSideCoefficients)
#
# 'FENESTRATIONSURFACE:DETAILED',
# (simple_surface, Surface_Type, Outside_Boundary_Condition)
# ----------------------------------------------------------
# ('WINDOW',  Window, None)
# ('DOOR', Door, None)


class NotImplementedError(Exception):
    """Exception Object"""

    pass


def bsdorigin(bsdobject, setto000=False):
    """return the origin of the surface"""
    # not yet implemented
    if setto000:
        return (0, 0, 0)
    else:
        raise NotImplementedError


def fsdorigin(fsdobject, setto000=False):
    """return the origin of the surface"""
    # not yet implemented
    if setto000:
        return (0, 0)
    else:
        raise NotImplementedError


def wallexterior(idf, bsdobject, deletebsd=True, setto000=False):
    """return an wall:exterior object if the  (buildingsurface:detailed) is
    an exterior wall"""
    # ('WALL:EXTERIOR', Wall, Outdoors)
    # test if it is an exterior wall
    if bsdobject.Surface_Type.upper() == "WALL":  # Surface_Type == wall
        if (
            bsdobject.Outside_Boundary_Condition.upper() == "OUTDOORS"
        ):  # Outside_Boundary_Condition == Outdoor
            simpleobject = idf.newidfobject("WALL:EXTERIOR")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Height = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def walladiabatic(idf, bsdobject, deletebsd=True, setto000=False):
    """return a wall:adiabatic if bsdobject (buildingsurface:detailed) is an
    adibatic wall"""
    # ('WALL:ADIABATIC',Wall, Adiabatic)
    # test if it is an adiabatic wall
    if bsdobject.Surface_Type.upper() == "WALL":  # Surface_Type == wall
        if (
            bsdobject.Outside_Boundary_Condition.upper() == "ADIABATIC"
        ):  # Outside_Boundary_Condition == Adiabatic
            simpleobject = idf.newidfobject("WALL:ADIABATIC")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Height = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def wallunderground(idf, bsdobject, deletebsd=True, setto000=False):
    """return a wall:underground if bsdobject (buildingsurface:detailed) is an
    underground wall"""
    # ('WALL:UNDERGROUND', Wall, s.startswith('Ground'))
    # test if it is an underground wall
    if bsdobject.Surface_Type.upper() == "WALL":  # Surface_Type == wall
        if bsdobject.Outside_Boundary_Condition.upper().startswith(
            "GROUND"
        ):  # Outside_Boundary_Condition startswith 'ground'
            simpleobject = idf.newidfobject("WALL:UNDERGROUND")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Height = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def wallinterzone(idf, bsdobject, deletebsd=True, setto000=False):
    """return an wall:interzone object if the bsd (buildingsurface:detailed)
    is an interaone wall"""
    # ('WALL:INTERZONE', Wall, Surface OR Zone OR OtherSideCoefficients)
    # test if it is an exterior wall
    if bsdobject.Surface_Type.upper() == "WALL":  # Surface_Type == wall
        if bsdobject.Outside_Boundary_Condition.upper() in (
            "SURFACE",
            "ZONE",
            "OtherSideCoefficients".upper(),
        ):
            simpleobject = idf.newidfobject("WALL:INTERZONE")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            obco = "Outside_Boundary_Condition_Object"
            simpleobject[obco] = bsdobject[obco]
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Height = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def roof(idf, bsdobject, deletebsd=True, setto000=False):
    """return an roof object if the bsd (buildingsurface:detailed) is
    a roof"""
    # ('ROOF', Roof, None or Outdoor)
    # test if it is aroof
    if bsdobject.Surface_Type.upper() == "ROOF":  # Surface_Type == roof
        if bsdobject.Outside_Boundary_Condition.upper() in (
            "OUTDOORS",
            "",
        ):  # Outside_Boundary_Condition == Outdoor
            simpleobject = idf.newidfobject("ROOF")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Width = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def ceilingadiabatic(idf, bsdobject, deletebsd=True, setto000=False):
    """return a ceiling:adiabatic if bsdobject (buildingsurface:detailed) is an
    adiabatic ceiling"""
    # ('CEILING:ADIABATIC', Ceiling, Adiabatic)
    # test if it is an adiabatic ceiling
    if bsdobject.Surface_Type.upper() == "CEILING":  # Surface_Type == ceiling
        if (
            bsdobject.Outside_Boundary_Condition.upper() == "ADIABATIC"
        ):  # Outside_Boundary_Condition == Adiabatic
            simpleobject = idf.newidfobject("CEILING:ADIABATIC")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Width = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


# ('CEILING:INTERZONE', Ceiling, Surface OR Zone)
def ceilinginterzone(idf, bsdobject, deletebsd=True, setto000=False):
    """return an ceiling:interzone object if the bsd (buildingsurface:detailed)
    is an interzone ceiling"""
    # ('WALL:INTERZONE', Wall, Surface OR Zone OR OtherSideCoefficients)
    # test if it is an exterior wall
    if bsdobject.Surface_Type.upper() == "CEILING":  # Surface_Type == ceiling
        if bsdobject.Outside_Boundary_Condition.upper() in (
            "SURFACE",
            "ZONE",
            "OtherSideCoefficients".upper(),
        ):
            simpleobject = idf.newidfobject("CEILING:INTERZONE")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            obco = "Outside_Boundary_Condition_Object"
            simpleobject[obco] = bsdobject[obco]
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Width = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def floorgroundcontact(idf, bsdobject, deletebsd=True, setto000=False):
    """return a wall:adiabatic if bsdobject (buildingsurface:detailed) is an
    adibatic wall"""
    # ('FLOOR:GROUNDCONTACT', Floor, s.startswith('Ground'))
    # test if it is an underground wall
    if bsdobject.Surface_Type.upper() == "FLOOR":  # Surface_Type == wall
        if bsdobject.Outside_Boundary_Condition.upper().startswith(
            "GROUND"
        ):  # Outside_Boundary_Condition startswith 'ground'
            simpleobject = idf.newidfobject("FLOOR:GROUNDCONTACT")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Width = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def flooradiabatic(idf, bsdobject, deletebsd=True, setto000=False):
    """return a floor:adiabatic if bsdobject (buildingsurface:detailed) is an
    adibatic floor"""
    # ('FLOOR:ADIABATIC', Floor, Adiabatic)
    # test if it is an adiabatic wall
    if bsdobject.Surface_Type.upper() == "FLOOR":  # Surface_Type == wall
        if (
            bsdobject.Outside_Boundary_Condition.upper() == "ADIABATIC"
        ):  # Outside_Boundary_Condition == Adiabatic
            simpleobject = idf.newidfobject("FLOOR:ADIABATIC")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Width = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def floorinterzone(idf, bsdobject, deletebsd=True, setto000=False):
    """return an floor:interzone object if the bsd (buildingsurface:detailed)
    is an interaone floor"""
    # ('FLOOR:INTERZONE', Floor, Surface OR Zone OR OtherSideCoefficients)
    # test if it is an exterior wall
    if bsdobject.Surface_Type.upper() == "FLOOR":  # Surface_Type == wall
        if bsdobject.Outside_Boundary_Condition.upper() in (
            "SURFACE",
            "ZONE",
            "OtherSideCoefficients".upper(),
        ):
            simpleobject = idf.newidfobject("FLOOR:INTERZONE")
            simpleobject.Name = bsdobject.Name
            simpleobject.Construction_Name = bsdobject.Construction_Name
            simpleobject.Zone_Name = bsdobject.Zone_Name
            obco = "Outside_Boundary_Condition_Object"
            simpleobject[obco] = bsdobject[obco]
            simpleobject.Azimuth_Angle = bsdobject.azimuth
            simpleobject.Tilt_Angle = bsdobject.tilt
            surforigin = bsdorigin(bsdobject, setto000=setto000)
            simpleobject.Starting_X_Coordinate = surforigin[0]
            simpleobject.Starting_Y_Coordinate = surforigin[1]
            simpleobject.Starting_Z_Coordinate = surforigin[2]
            simpleobject.Length = bsdobject.width
            simpleobject.Width = bsdobject.height
            if deletebsd:
                idf.removeidfobject(bsdobject)
            return simpleobject
    return None


def window(idf, fsdobject, deletebsd=True, setto000=False):
    """return an window object if the fsd (fenestrationsurface:detailed) is
    a window"""
    # ('WINDOW',  Window, None)
    if fsdobject.Surface_Type.upper() == "WINDOW":  # Surface_Type == w
        simpleobject = idf.newidfobject("WINDOW")
        simpleobject.Name = fsdobject.Name
        simpleobject.Construction_Name = fsdobject.Construction_Name
        simpleobject.Building_Surface_Name = fsdobject.Building_Surface_Name
        simpleobject.Shading_Control_Name = fsdobject.Shading_Control_Name
        simpleobject.Frame_and_Divider_Name = fsdobject.Frame_and_Divider_Name
        simpleobject.Multiplier = fsdobject.Multiplier
        surforigin = fsdorigin(fsdobject, setto000=setto000)
        simpleobject.Starting_X_Coordinate = surforigin[0]
        simpleobject.Starting_Z_Coordinate = surforigin[1]
        simpleobject.Length = fsdobject.width
        simpleobject.Height = fsdobject.height
        if deletebsd:
            idf.removeidfobject(fsdobject)
        return simpleobject
    return None


def door(idf, fsdobject, deletebsd=True, setto000=False):
    """return an door object if the fsd (fenestrationsurface:detailed) is
    a door"""
    # ('DOOR', Door, None)
    # test if it is aroof
    if fsdobject.Surface_Type.upper() == "DOOR":  # Surface_Type == w
        simpleobject = idf.newidfobject("DOOR")
        simpleobject.Name = fsdobject.Name
        simpleobject.Construction_Name = fsdobject.Construction_Name
        simpleobject.Building_Surface_Name = fsdobject.Building_Surface_Name
        simpleobject.Multiplier = fsdobject.Multiplier
        surforigin = fsdorigin(fsdobject, setto000=setto000)
        simpleobject.Starting_X_Coordinate = surforigin[0]
        simpleobject.Starting_Z_Coordinate = surforigin[1]
        simpleobject.Length = fsdobject.width
        simpleobject.Height = fsdobject.height
        if deletebsd:
            idf.removeidfobject(fsdobject)
        return simpleobject
    return None


def glazeddoor(idf, fsdobject, deletebsd=True, setto000=False):
    """return an glazeddoor object if the fsd (fenestrationsurface:detailed) is
    a glassdoor"""
    # ('WINDOW',  glassdoor, None)
    # test if it is glassdoor
    if fsdobject.Surface_Type.upper() == "GLASSDOOR":
        simpleobject = idf.newidfobject("GLAZEDDOOR")
        simpleobject.Name = fsdobject.Name
        simpleobject.Construction_Name = fsdobject.Construction_Name
        simpleobject.Building_Surface_Name = fsdobject.Building_Surface_Name
        simpleobject.Shading_Control_Name = fsdobject.Shading_Control_Name
        simpleobject.Frame_and_Divider_Name = fsdobject.Frame_and_Divider_Name
        simpleobject.Multiplier = fsdobject.Multiplier
        surforigin = fsdorigin(fsdobject, setto000=setto000)
        simpleobject.Starting_X_Coordinate = surforigin[0]
        simpleobject.Starting_Z_Coordinate = surforigin[1]
        simpleobject.Length = fsdobject.width
        simpleobject.Height = fsdobject.height
        if deletebsd:
            idf.removeidfobject(fsdobject)
        return simpleobject
    return None


def simplesurface(idf, bsd, deletebsd=True, setto000=False):
    """convert a bsd (buildingsurface:detailed) into a simple surface"""
    funcs = (
        wallexterior,
        walladiabatic,
        wallunderground,
        wallinterzone,
        roof,
        ceilingadiabatic,
        ceilinginterzone,
        floorgroundcontact,
        flooradiabatic,
        floorinterzone,
    )
    for func in funcs:
        surface = func(idf, bsd, deletebsd=deletebsd, setto000=setto000)
        if surface:
            return surface
    return None


def simplefenestration(idf, fsd, deletebsd=True, setto000=False):
    """convert a bsd (fenestrationsurface:detailed) into a simple
    fenestrations"""
    funcs = (window, door, glazeddoor)
    for func in funcs:
        fenestration = func(idf, fsd, deletebsd=deletebsd, setto000=setto000)
        if fenestration:
            return fenestration
    return None



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

# Find the intersection between two lines
# V = (1/6)*|(a-d).((b-d)x(c-d))|

"""
Find the intersection between two lines
V = (1/6)*|(a-d).((b-d)x(c-d))|
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import numpy as np


def vol_tehrahedron(poly):
    """volume of a irregular tetrahedron"""
    a_pnt = np.array(poly[0])
    b_pnt = np.array(poly[1])
    c_pnt = np.array(poly[2])
    d_pnt = np.array(poly[3])
    return abs(np.dot((a_pnt - d_pnt), np.cross((b_pnt - d_pnt), (c_pnt - d_pnt))) / 6)


# poly = [(0.0, 0.0, 0.0), (1.0, 0.0, 0.0), (1.0, 1.0, 0.0), (0.0, 0.0, 1.0)]


def central_p(poly1, poly2):
    """central point of a prism"""
    central_point = np.array([0.0, 0.0, 0.0])
    for i in range(len(poly1)):
        central_point += np.array(poly1[i]) + np.array(poly2[i])
    return central_point / (len(poly1)) / 2


# poly1 = [(0.0, 0.0, 0.0), (1.0, 0.0, 0.0), (1.0, 1.0, 0.0), (0, 1.0, 0.0)]
# poly2 = [(0.0, 0.0, 1), (1.0, 0.0, 1), (1.0, 1.0, 1), (0, 1.0, 1)]


def vol_zone(poly1, poly2):
    """"volume of a zone defined by two polygon bases """
    c_point = central_p(poly1, poly2)
    c_point = (c_point[0], c_point[1], c_point[2])
    vol_therah = 0
    num = len(poly1)
    for i in range(num - 2):
        # the upper part
        tehrahedron = [c_point, poly1[0], poly1[i + 1], poly1[i + 2]]
        vol_therah += vol_tehrahedron(tehrahedron)
        # the bottom part
        tehrahedron = [c_point, poly2[0], poly2[i + 1], poly2[i + 2]]
        vol_therah += vol_tehrahedron(tehrahedron)
    # the middle part
    for i in range(num - 1):
        tehrahedron = [c_point, poly1[i], poly2[i], poly2[i + 1]]
        vol_therah += vol_tehrahedron(tehrahedron)
        tehrahedron = [c_point, poly1[i], poly1[i + 1], poly2[i]]
        vol_therah += vol_tehrahedron(tehrahedron)
    tehrahedron = [c_point, poly1[num - 1], poly2[num - 1], poly2[0]]
    vol_therah += vol_tehrahedron(tehrahedron)
    tehrahedron = [c_point, poly1[num - 1], poly1[0], poly2[0]]
    vol_therah += vol_tehrahedron(tehrahedron)
    return vol_therah



import pytest


@pytest.mark.parametrize("key", ["BUILDING", "Building", "building", "BuIlDiNg"])
def test_get_and_set(base_idf, key):
    idf = base_idf
    idf.newidfobject(key, Name="Building")
    buildings = idf.idfobjects["building"]
    assert len(buildings) == 1


@pytest.mark.parametrize("key", ["BUILDING", "Building", "building", "BuIlDiNg"])
def test_contains(base_idf, key):
    idf = base_idf
    idf.newidfobject(key, Name="Building")
    assert key in idf.idfobjects


@pytest.mark.parametrize("key", ["BUILDING", "Building", "building", "BuIlDiNg"])
def test_del(base_idf, key):
    idf = base_idf
    idf.newidfobject(key, Name="Building")
    assert key in idf.idfobjects
    del idf.idfobjects["building"]
    assert key not in idf.idfobjects



# Copyright (c) 2016 Jamie Bull
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
Integration tests for features in eppy.runner related to running EnergyPlus
from Eppy. These tests will fail unless run on a system with EnergyPlus
installed in the default location.

"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from glob import glob
import multiprocessing
import os
import re
import shutil

import pytest
from importlib import reload

from eppy import modeleditor
from eppy.pytest_helpers import do_integration_tests
from eppy.runner.run_functions import install_paths, EnergyPlusRunError
from eppy.runner.run_functions import multirunner
from eppy.runner.run_functions import run
from eppy.runner.run_functions import runIDFs


def versiontuple(vers):
    """version tuple"""
    return tuple([int(num) for num in vers.split("-")])


THIS_DIR = os.path.dirname(os.path.abspath(__file__))

RESOURCES_DIR = os.path.join(THIS_DIR, os.pardir, "eppy", "resources")

IDD_FILES = os.path.join(RESOURCES_DIR, "iddfiles")
IDF_FILES = os.path.join(RESOURCES_DIR, "idffiles")
try:
    VERSION = os.environ["ENERGYPLUS_INSTALL_VERSION"]  # used in CI files
except KeyError:
    VERSION = "8-9-0"  # current default for integration tests on local system
TEST_IDF = "V{}/smallfile.idf".format(VERSION[:3].replace("-", "_"))
TEST_EPW = "USA_CA_San.Francisco.Intl.AP.724940_TMY3.epw"
TEST_IDD = "Energy+V{}.idd".format(VERSION.replace("-", "_"))
TEST_OLD_IDD = "Energy+V7_2_0.idd"

eplus_exe, eplus_weather = install_paths(VERSION, os.path.join(IDD_FILES, TEST_IDD))


def has_severe_errors(results="run_outputs"):
    """Check for severe errors in the eplusout.end file."""
    end_filename = glob("{}/*.end".format(results))[0]
    with open(os.path.join(end_filename), "r") as end_file:
        end_txt = end_file.read()
    num_severe = int(re.findall(r" (\d*) Severe Error", end_txt)[0])
    return num_severe > 0


def test_version_reader():
    """Test that get the expected idd_version when reading an IDF/IDD."""
    # We need to reload modeleditor since the IDF class may have had an IDD
    # which causes problems.
    # https://stackoverflow.com/questions/437589/how-do-i-unload-reload-a-python-module
    reload(modeleditor)
    iddfile = os.path.join(IDD_FILES, TEST_IDD)
    fname1 = os.path.join(IDF_FILES, TEST_IDF)
    modeleditor.IDF.setiddname(iddfile, testing=True)
    idf = modeleditor.IDF(fname1, TEST_EPW)
    ep_version = idf.idd_version
    assert ep_version == versiontuple(VERSION)
    ep_version = modeleditor.IDF.idd_version
    assert ep_version == versiontuple(VERSION)
    # reload(modeleditor)


@pytest.mark.skipif(
    not do_integration_tests(), reason="$EPPY_INTEGRATION env var not set"
)
class TestEnvironment(object):

    """
    Test that the environment has been correctly set up with EnergyPlus
    in the default location.

    """

    def test_thisdir_exists(self):
        """Make sure we are starting from the correct path."""
        assert os.path.isdir(THIS_DIR)

    def test_iddfiles_exists(self):
        """Test the IDD files are where we expect them."""
        assert os.path.isdir(IDD_FILES)

    def test_idffiles_exists(self):
        """Test the test IDF files are where we expect them."""
        assert os.path.isdir(IDF_FILES)

    def test_epw_exists(self):
        """Test the test EPW file is where we expect it to be."""
        f = os.path.join(eplus_weather, TEST_EPW)
        assert os.path.isfile(f)

    def test_idf_exists(self):
        """Test the test IDF file is where we expect it to be."""
        f = os.path.join(IDF_FILES, TEST_IDF)
        assert os.path.isfile(f)

    def test_idd_exists(self):
        """Test the test IDD file is where we expect it to be."""
        f = os.path.join(IDD_FILES, TEST_IDD)
        assert os.path.isfile(f)

    def test_old_idd_exists(self):
        """Test the test old IDD file is where we expect it to be."""
        f = os.path.join(IDD_FILES, TEST_OLD_IDD)
        assert os.path.isfile(f)


@pytest.mark.skipif(
    not do_integration_tests(), reason="$EPPY_INTEGRATION env var not set"
)
class TestRunFunction(object):

    """Tests for simple running of EnergyPlus from Eppy."""

    def setup(self):
        """Tidy up just in case anything is left from previous test runs."""
        # reload(modeleditor)
        os.chdir(THIS_DIR)
        shutil.rmtree("test_results", ignore_errors=True)
        shutil.rmtree("run_outputs", ignore_errors=True)

    def teardown(self):
        """Tidy up after tests."""
        os.chdir(THIS_DIR)
        shutil.rmtree("test_results", ignore_errors=True)
        shutil.rmtree("run_outputs", ignore_errors=True)
        # reload(modeleditor)

    def test_run_abs_paths(self):
        """
        Test that running works with absolute paths.
        Fails if no results are produced.

        """
        fname1 = os.path.join(IDF_FILES, TEST_IDF)
        epw = os.path.join(eplus_weather, TEST_EPW)
        run(fname1, epw, output_directory="test_results", ep_version=VERSION)
        assert len(os.listdir("test_results")) > 0
        for f in os.listdir("test_results"):
            assert os.path.isfile(os.path.join("test_results", f))

    def test_run_weather_name_only(self):
        """
        Test that running works with the name of a weather file that is
        in the Weather subdirectory of the EnergyPlus install directory.
        Fails if no results are produced.

        """
        fname1 = os.path.join(IDF_FILES, TEST_IDF)
        run(fname1, TEST_EPW, output_directory="test_results", ep_version=VERSION)
        assert len(os.listdir("test_results")) > 0
        for f in os.listdir("test_results"):
            assert os.path.isfile(os.path.join("test_results", f))

    def test_run_missing_file_raises_error(self, capfd):
        """
        Test that a missing file produces the expected warning to std
        out.
        Fails if error message is not as expected.

        """
        fname1 = os.path.join(IDF_FILES, "XXXXXXX_fake_file.idf")
        with pytest.raises(EnergyPlusRunError):
            run(fname1, TEST_EPW, output_directory="test_results", ep_version=VERSION)
            out, _err = capfd.readouterr()
            assert "ERROR: Could not find input data file:" in out


@pytest.mark.skipif(
    not do_integration_tests(), reason="$EPPY_INTEGRATION env var not set"
)
class TestIDFRunner(object):

    """Tests for running EnergyPlus from an IDF object."""

    def setup(self):
        """Tidy up anything left from previous runs. Get an IDF object to run."""
        # reload(modeleditor)
        shutil.rmtree(os.path.join(THIS_DIR, "run_outputs"), ignore_errors=True)

        self.expected_files = [
            "eplusout.audit",
            "eplusout.bnd",
            "eplusout.eio",
            "eplusout.end",
            "eplusout.err",
            "eplusout.eso",
            "eplusout.mdd",
            "eplusout.mtd",
            "eplusout.rdd",
            "eplusout.shd",
            "eplustbl.htm",
            "sqlite.err",
        ]
        self.expected_files_suffix_C = [
            "eplus.audit",
            "eplus.mdd",
            "eplus.err",
            "eplusSqlite.err",
            "eplus.eio",
            "eplusTable.htm",
            "eplus.shd",
            "eplus.mtd",
            "eplus.bnd",
            "eplus.eso",
            "eplus.rdd",
            "eplus.end",
        ]
        self.expected_files_suffix_D = [
            "eplus.audit",
            "eplus.mdd",
            "eplus-sqlite.err",
            "eplus-table.htm",
            "eplus.err",
            "eplus.eio",
            "eplus.bnd",
            "eplus.shd",
            "eplus.mtd",
            "eplus.end",
            "eplus.eso",
            "eplus.rdd",
        ]

    def teardown(self):
        """Destroy temp dir, reset working directory, destroy outputs."""
        os.chdir(THIS_DIR)
        shutil.rmtree("run_outputs", ignore_errors=True)
        shutil.rmtree("other_run_outputs", ignore_errors=True)
        shutil.rmtree("test_results", ignore_errors=True)
        # reload(modeleditor)
        for f in {"eplusout.end", "eplusout.err", "in.idf"}:
            try:
                os.remove(os.path.join(THIS_DIR, f))
            except OSError:  # file was not generated on this run
                pass

    def num_rows_in_csv(self, results="./run_outputs"):
        """Check readvars outputs the expected number of rows."""
        with open(os.path.join(results, "eplusout.csv"), "r") as csv_file:
            return len(csv_file.readlines())

    def test_run(self, test_idf):
        """
        End to end test of idf.run function.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.run(output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        assert set(files) == set(self.expected_files)

    def test_run_readvars(self, test_idf):
        """
        End to end test of idf.run function with readvars set True.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.run(readvars=True, output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend(["eplusout.rvaudit", "eplusout.csv"])
        assert set(files) == set(self.expected_files)

    def test_run_annual(self, test_idf):
        """
        End to end test of idf.run function with annual set True.
        Fails on incorrect size of CSV output, severe errors or
        unexpected/missing output files.

        """
        test_idf.idfobjects["RUNPERIOD"][0].End_Month = 1
        test_idf.run(annual=True, readvars=True, output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend(["eplusout.rvaudit", "eplusout.csv"])
        assert set(files) == set(self.expected_files)
        assert self.num_rows_in_csv() == 35041  # 24 * 365 * 4 + 1 header row

    def test_run_output_directory(self, test_idf):
        """
        End to end test of idf.run function with a specific output dir set.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.run(output_directory="other_run_outputs")
        assert not has_severe_errors("other_run_outputs")
        files = os.listdir("other_run_outputs")
        self.expected_files.extend([])
        assert set(files) == set(self.expected_files)

    def test_run_design_day(self, test_idf):
        """
        End to end test of idf.run function with design_day flag set True.
        Fails on incorrect size of CSV output, severe errors or
        unexpected/missing output files.

        """
        test_idf.run(design_day=True, readvars=True, output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend(["eplusout.rvaudit", "eplusout.csv"])
        assert set(files) == set(self.expected_files)
        assert self.num_rows_in_csv() == 193  # 24 * 8 + 1 header row

    def test_run_epmacro(self, test_idf):
        """
        End to end test of idf.run function with epmacro flag set True.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.run(epmacro=True, output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend(["eplusout.epmdet", "eplusout.epmidf"])
        assert set(files) == set(self.expected_files)

    def test_run_expandobjects(self, test_idf):
        """
        End to end test of idf.run function with expandobjects flag set to
        True.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.newidfobject(
            "HVACTEMPLATE:THERMOSTAT",
            Name="TestThermostat",
            Cooling_Setpoint_Schedule_Name="",
            Heating_Setpoint_Schedule_Name="",
            Constant_Cooling_Setpoint=25,
            Constant_Heating_Setpoint=21,
        )
        test_idf.run(expandobjects=True, output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend(["eplusout.expidf"])
        assert set(files) == set(self.expected_files)

    def test_run_expandobjects_no_flag(self, test_idf):
        """
        End to end test of idf.run function with expandobjects flag unset.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.newidfobject(
            "HVACTEMPLATE:THERMOSTAT",
            Name="TestThermostat",
            Cooling_Setpoint_Schedule_Name="",
            Heating_Setpoint_Schedule_Name="",
            Constant_Cooling_Setpoint=25,
            Constant_Heating_Setpoint=21,
        )
        test_idf.run(output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend(["eplusout.expidf"])
        assert set(files) == set(self.expected_files)

    def test_run_output_prefix(self, test_idf):
        """
        End to end test of idf.run function with output prefix set.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.run(output_prefix="test", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        prefixed_files = [f.replace("eplus", "test") for f in self.expected_files]
        assert set(files) == set(prefixed_files)

    def test_run_output_suffix_L(self, test_idf):
        """
        End to end test of idf.run function with output suffix set.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.run(output_suffix="L", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        assert set(files) == set(self.expected_files)

    def test_run_output_suffix_C(self, test_idf):
        """
        End to end test of idf.run function with output suffix set.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.run(output_suffix="C", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        assert set(files) == set(self.expected_files_suffix_C)

    def test_run_output_suffix_D(self, test_idf):
        """
        End to end test of idf.run function with output suffix set.
        Fails on severe errors or unexpected/missing output files.

        """
        test_idf.run(output_suffix="D", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        assert set(files) == set(self.expected_files_suffix_D)

    @pytest.mark.skipif(
        versiontuple(VERSION) >= (8, 9, 0),
        reason="Only compiled IDD is used from v8.9.0",
    )
    def test_run_IDD(self, test_idf):
        """
        End to end test of idf.run function with a different IDD set.
        We use an old IDD here since it throws an error we can see. Real uses
        of this option would include using an IDD with extra fields added
        extensible fields to allow larger objects.
        Fails if the expected IDD version is not listed in the error file.

        """
        other_idd = os.path.join(IDD_FILES, TEST_OLD_IDD)
        test_idf.run(idd=other_idd, output_directory="run_outputs")
        with open("run_outputs/eplusout.err", "r") as errors:
            assert "IDD_Version 8.1.0.009" in errors.readline()

    def test_version(self, capfd, test_idf):
        """
        End to end test of idf.run function with the version flag set True.
        Fails if the expected EnergyPlus version number is not returned.

        """
        test_idf.run(version=True)
        out, _err = capfd.readouterr()

        expected_version = VERSION.replace("-", ".")
        version_string = "EnergyPlus, Version {}".format(expected_version)

        assert out.strip().startswith(version_string)

    def test_help(self, capfd, test_idf):
        """
        Test of calling the `help` built-in function on an IDF object.
        Fails if the expected help output is not returned.

        """
        help(test_idf.run)
        out, _err = capfd.readouterr()
        expected = "Help on method run in module eppy.modeleditor:"

        assert out.strip().startswith(expected)

    def test_verbose(self, capfd, test_idf):
        """
        End to end test of idf.run function with the version flag set True.
        Fails on severe errors or unexpected/missing output files.
        Fails if no output received from EnergyPlus.

        """
        test_idf.run(verbose="v", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend([])
        assert set(files) == set(self.expected_files)
        out, _err = capfd.readouterr()
        assert len(out) > 0

    def test_Verbose(self, capfd, test_idf):
        """
        End to end test of idf.run function with the version flag set True.
        Fails on severe errors or unexpected/missing output files.
        Fails if no output received from EnergyPlus.

        """
        test_idf.run(verbose="V", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend([])
        assert set(files) == set(self.expected_files)
        out, _err = capfd.readouterr()
        assert len(out) > 0

    def test_quiet(self, capfd, test_idf):
        """
        End to end test of idf.run function with the version flag set True.
        Fails on severe errors or unexpected/missing output files.
        Fails if output received from EnergyPlus.

        """
        test_idf.run(verbose="q", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend([])
        assert set(files) == set(self.expected_files)
        out, _err = capfd.readouterr()
        assert len(out) == 0

    def test_Quiet(self, capfd, test_idf):
        """
        End to end test of idf.run function with the version flag set True.
        Fails on severe errors or unexpected/missing output files.
        Fails if output received from EnergyPlus.

        """
        test_idf.run(verbose="Q", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend([])
        assert set(files) == set(self.expected_files)
        out, _err = capfd.readouterr()
        assert len(out) == 0

    def test_silent(self, capfd, test_idf):
        """
        End to end test of idf.run function with the version flag set True.
        Fails on severe errors or unexpected/missing output files.
        Fails if output received from EnergyPlus.
        """
        test_idf.run(verbose="s", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend([])
        assert set(files) == set(self.expected_files)
        out, _err = capfd.readouterr()
        assert len(out) == 0

    def test_Silent(self, capfd, test_idf):
        """
        End to end test of idf.run function with the version flag set True.
        Fails on severe errors or unexpected/missing output files.
        Fails if output received from EnergyPlus.
        """
        test_idf.run(verbose="S", output_directory="run_outputs")
        assert not has_severe_errors()
        files = os.listdir("run_outputs")
        self.expected_files.extend([])
        assert set(files) == set(self.expected_files)
        out, _err = capfd.readouterr()
        assert len(out) == 0

    def test_reset_cwd_on_failure(self, capfd, test_idf):
        cwd = os.getcwd()
        test_idf.idfobjects["RUNPERIOD"][0].Begin_Month = "Spamuary"
        with pytest.raises(EnergyPlusRunError):
            test_idf.run(output_directory="run_outputs")
        out, _err = capfd.readouterr()
        assert "FATAL" in out
        assert os.getcwd() == cwd

    def test_exception_message(self, test_idf):
        test_idf.newidfobject("HVACTemplate:Thermostat", Name="Thermostat")
        with pytest.raises(EnergyPlusRunError) as exc_info:
            test_idf.run(output_directory="run_outputs")
        assert "ExpandObjects program" in str(exc_info.value)


@pytest.mark.skipif(
    not do_integration_tests(), reason="$EPPY_INTEGRATION env var not set"
)
class TestMultiprocessing(object):

    """Tests for running multiple EnergyPlus jobs simultaneously."""

    def setup(self):
        """Clear out any results from previous tests."""
        # reload(modeleditor)
        os.chdir(THIS_DIR)
        shutil.rmtree("multirun_outputs", ignore_errors=True)
        self.expected_files = [
            "eplusout.audit",
            "eplusout.bnd",
            "eplusout.eio",
            "eplusout.end",
            "eplusout.err",
            "eplusout.eso",
            "eplusout.mdd",
            "eplusout.mtd",
            "eplusout.rdd",
            "eplusout.shd",
            "eplustbl.htm",
            "sqlite.err",
        ]

    def teardown(self):
        """Remove the multiprocessing results folders."""
        for results_dir in glob("results_*"):
            shutil.rmtree(results_dir)
        shutil.rmtree("test_results", ignore_errors=True)
        shutil.rmtree("run_outputs", ignore_errors=True)
        # reload(modeleditor)

    def test_sequential_run(self):
        """
        Test that we can run a sequence of runs using the signature:
            run([idf_path, epw], kwargs)
        Fails if expected output files are not in the expected output
        directories.

        """
        fname1 = os.path.join(IDF_FILES, TEST_IDF)
        runs = []
        for i in range(2):
            kwargs = {"output_directory": "results_%s" % i, "ep_version": VERSION}
            runs.append([[fname1, TEST_EPW], kwargs])
        for r in runs:
            run(*r[0], **r[1])
        for results_dir in glob("results_*"):
            assert not has_severe_errors(results_dir)
            files = os.listdir(results_dir)
            assert set(files) == set(self.expected_files)

    def test_multiprocess_run(self):
        """
        Test that we can run a list of runs in parallel using the signature:
            run([idf_path, epw], kwargs)
        Fails if expected output files are not in the expected output
        directories.

        """
        fname1 = os.path.join(IDF_FILES, TEST_IDF)
        runs = []
        ep_version = "-".join(str(x) for x in modeleditor.IDF.idd_version[:3])
        assert ep_version == VERSION
        for i in range(2):
            kwargs = {"output_directory": "results_%s" % i, "ep_version": ep_version}
            runs.append([[fname1, TEST_EPW], kwargs])
        pool = multiprocessing.Pool(2)
        pool.map(multirunner, runs)
        pool.close()

    def test_multiprocess_run_IDF(self):
        """
        Test that we can run a sequence of runs using the signature:
            runIDFs([[IDF, kwargs],...], num_CPUs)
        Fails if expected output files are not in the expected output
        directories.

        """
        iddfile = os.path.join(IDD_FILES, TEST_IDD)
        fname1 = os.path.join(IDF_FILES, TEST_IDF)
        modeleditor.IDF.setiddname(open(iddfile, "r"), testing=True)
        ep_version = "-".join(str(x) for x in modeleditor.IDF.idd_version[:3])
        assert ep_version == VERSION
        runs = []
        for i in range(4):
            runs.append(
                [
                    modeleditor.IDF(open(fname1, "r"), TEST_EPW),
                    {"output_directory": "results_%i" % i, "ep_version": ep_version},
                ]
            )
        num_CPUs = 2
        runIDFs(runs, num_CPUs)

        num_CPUs = -1
        runIDFs(runs, num_CPUs)

    def test_multiprocess_run_IDF_from_generator(self):
        """
        Test that we can run a sequence passed as a generator of runs using
        the signature:
            runIDFs(([IDF, kwargs],...), num_CPUs)
        Fails if expected output files are not in the expected output
        directories.

        """
        iddfile = os.path.join(IDD_FILES, TEST_IDD)
        fname1 = os.path.join(IDF_FILES, TEST_IDF)
        modeleditor.IDF.setiddname(open(iddfile, "r"), testing=True)
        ep_version = "-".join(str(x) for x in modeleditor.IDF.idd_version[:3])
        assert ep_version == VERSION
        runs = (
            (
                modeleditor.IDF(open(fname1, "r"), TEST_EPW),
                {"output_directory": "results_%i" % i, "ep_version": ep_version},
            )
            for i in range(4)
        )
        num_CPUs = 2
        runIDFs(runs, num_CPUs)

        num_CPUs = -1
        runIDFs(runs, num_CPUs)



# Copyright (c) 2019 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
# -*- coding: utf-8 -*-

"""Top-level package for eppy."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

__author__ = """Santosh Philip"""
__email__ = "santosh@noemail.com"
__version__ = "0.5.57"


from io import StringIO
import eppy


def newidf(version=None):
    """open a new idf file

    easy way to open a new idf file for particular version. Works only id Energyplus of that version is installed.

    Parameters
    ----------
    version: string
        version of the new file you want to create. Will work only if this version of Energyplus has been installed.

    Returns
    -------
    idf
       file of type eppy.modelmake.IDF
    """  # noqa: E501
    if not version:
        version = "8.9"
    import eppy.easyopen as easyopen

    idfstring = "  Version,{};".format(str(version))
    fhandle = StringIO(idfstring)
    return easyopen.easyopen(fhandle)


def openidf(fname, idd=None, epw=None):
    """automatically set idd and open idf file. Uses version from idf to set correct idd
    It will work under the following circumstances:

    - the IDF file should have the VERSION object.
    - Needs  the version of EnergyPlus installed that matches the IDF version.
    - Energyplus should be installed in the default location.

    Parameters
    ----------
    fname : str, StringIO or IOBase
        Filepath IDF file,
        File handle of IDF file open to read
        StringIO with IDF contents within
    idd : str, StringIO or IOBase
        This is an optional argument. easyopen will find the IDD without this arg
        Filepath IDD file,
        File handle of IDD file open to read
        StringIO with IDD contents within
    epw : str
        path name to the weather file. This arg is needed to run EneryPlus from eppy.
    """
    import eppy.easyopen as easyopen

    return easyopen.easyopen(fname, idd=idd, epw=epw)



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""read the idf file by just parsing the text"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import eppy.modeleditor as modeleditor
from eppy.modeleditor import IDF


def nocomment(astr, com="!"):
    """
    just like the comment in python.
    removes any text after the phrase 'com'
    """
    alist = astr.splitlines()
    for i in range(len(alist)):
        element = alist[i]
        pnt = element.find(com)
        if pnt != -1:
            alist[i] = element[:pnt]
    return "\n".join(alist)


def _tofloat(num):
    """to float"""
    try:
        return float(num)
    except ValueError:
        return num


def idf2txt(txt):
    """convert the idf text to a simple text"""
    astr = nocomment(txt)
    objs = astr.split(";")
    objs = [obj.split(",") for obj in objs]
    objs = [[line.strip() for line in obj] for obj in objs]
    objs = [[_tofloat(line) for line in obj] for obj in objs]
    objs = [tuple(obj) for obj in objs]
    objs.sort()

    lst = []
    for obj in objs:
        for field in obj[:-1]:
            lst.append("%s," % (field,))
        lst.append("%s;\n" % (obj[-1],))

    return "\n".join(lst)


def idfreadtest(iddhandle, idfhandle1, idfhandle2, verbose=False, save=False):
    """compare the results of eppy reader and simple reader"""
    # read using eppy:
    try:
        IDF.setiddname(iddhandle)
    except modeleditor.IDDAlreadySetError:
        # idd has already been set
        pass
    idf = IDF(idfhandle1)
    idfstr = idf.idfstr()
    idfstr = idf2txt(idfstr)
    # -
    # do a simple read
    simpletxt = idfhandle2.read()
    try:
        simpletxt = simpletxt.decode("ISO-8859-2")
    except AttributeError:
        pass
    simpletxt = idf2txt(simpletxt)
    # -
    if save:
        open("simpleread.idf", "w").write(idfstr)
        open("eppyread.idf", "w").write(simpletxt)
    # do the compare
    lines1 = idfstr.splitlines()
    lines2 = simpletxt.splitlines()
    for i, (line1, line2) in enumerate(zip(lines1, lines2)):
        if line1 != line2:
            # test if it is a mismatch in number format
            try:
                line1 = float(line1[:-1])
                line2 = float(line2[:-1])
                if line1 != line2:
                    if verbose:
                        print()
                        print("%s- : %s" % (i, line1))
                        print("%s- : %s" % (i, line2))
                    return False
            except ValueError:
                if verbose:
                    print()
                    print("%s- : %s" % (i, line1))
                    print("%s- : %s" % (i, line2))
                return False
    return True



# Copyright (c) 2016 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""integration tests for eppy"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from itertools import product
import os
import shutil
import sys

from eppy.modeleditor import IDF
from eppy.pytest_helpers import IDD_FILES
from eppy.pytest_helpers import INTEGRATION_FILES
from eppy.pytest_helpers import PATH_TO_EPPY
from eppy.pytest_helpers import do_integration_tests
import pytest


sys.path.append(PATH_TO_EPPY)


def getversion(idf):
    """return the version number"""
    versions = idf.idfobjects["VERSION"]
    return versions[0].Version_Identifier


def setversion(idf, newversion):
    """set the version number"""
    versions = idf.idfobjects["VERSION"]
    versions[0].Version_Identifier = newversion


@pytest.mark.skipif(
    not do_integration_tests(), reason="$EPPY_INTEGRATION env var not set"
)
class TestModeleditorIntegration:
    def setup(self):
        """Set the IDD and file paths, and make a copy of the original file."""
        iddfile = os.path.join(IDD_FILES, "Energy+V7_2_0.idd")
        IDF.setiddname(iddfile, testing=True)

        self.origfile = os.path.join(INTEGRATION_FILES, "origfile.idf")

        # set tempfile names
        self.startfile = os.path.join(INTEGRATION_FILES, "startfile.idf")
        self.saveasfile = os.path.join(INTEGRATION_FILES, "saveas.idf")
        self.copyfile = os.path.join(INTEGRATION_FILES, "savecopy.idf")

        # make a copy of test file
        shutil.copy(self.origfile, self.startfile)

    def teardown(self):
        """Clear up temp files so we don't accidentally check against them."""
        tempfiles = [self.startfile, self.saveasfile, self.copyfile]
        for f in tempfiles:
            try:
                os.remove(f)
            except OSError:
                pass

    def test_version_number(self):
        """Test opening an IDF file and checking version number.

        Fails if the version number is not the expected starting version number.

        """
        idf = IDF(self.startfile)

        result = getversion(idf)
        expected = "7.3"

        assert result == expected

    def test_save(self):
        """Test save with a changed version number.

        Fails if the version number has not been changed.

        """
        idf = IDF(self.startfile)
        setversion(idf, "7.4")
        idf.save()
        idf2 = IDF(self.startfile)
        assert idf is not idf2  # be sure we don't just have an in-memory copy

        result = getversion(idf2)
        expected = "7.4"

        assert result == expected

    def test_save_as(self):
        """Test saveas with a changed filename.

        IDF.saveas(fname) changes the filename. The next save should save with
        new name.

        Fails if the filename isn't changed on the file with the new name.
        """
        idf = IDF(self.startfile)
        idf.saveas(self.saveasfile)  # sets the new filename
        setversion(idf, "7.4")
        idf.save()  # should also save with the new filename
        idf2 = IDF(self.saveasfile)

        result = getversion(idf2)
        expected = "7.4"

        assert result == expected

    def test_save_copy(self):
        """Test savecopy with a new filename.

        IDF.savecopy(fname) doesn't change the filename. The next save should
        save with the original name.

        Fails if on a following save, the copy is changed.

        """
        idf = IDF(self.startfile)
        idf.savecopy(self.copyfile)
        setversion(idf, "7.5")
        idf.save()
        idf2 = IDF(self.copyfile)

        result = getversion(idf2)
        expected = "7.3"  # unchanged since version was changed after savecopy

        assert result == expected

        idf3 = IDF(self.startfile)
        result = getversion(idf3)
        expected = "7.5"  # should be changed in the original file

        assert result == expected

    def test_lineendings(self):
        """Test lineendings are set correctly on each platform."""
        idf = IDF(self.startfile)
        idf.save(lineendings="windows")
        with open(self.startfile, "rb") as sf:
            txt = sf.read()
        print(txt.count(b"\r\n"))
        lines = txt.splitlines()
        numlines = len(lines)
        assert numlines == txt.count(b"\r\n") + 1  # no CR on last line
        idf.save(lineendings="unix")
        with open(self.origfile, "rb") as of:
            txt = of.read()
        lines = txt.splitlines()
        numlines = len(lines)
        assert numlines == txt.count(b"\n") + 1  # no CR on last line

    def test_save_with_lineendings_and_encodings(self):
        """
        Test the IDF.save() function with combinations of encodings and line
        endings.

        """
        idf = IDF(self.startfile)
        lineendings = ("windows", "unix", "default")
        encodings = ("ascii", "latin-1", "UTF-8")

        for le, enc in product(lineendings, encodings):
            idf.save(lineendings=le, encoding=enc)

            with open(self.startfile, "rb") as sf:
                result = sf.read()
            if le == "windows":
                assert b"\r\n" in result
            elif le == "unix":
                assert b"\r\n" not in result
            elif le == "default":
                assert os.linesep.encode(enc) in result



"""py.test for walk_hvac.py"""

# Copyright (c) 2016 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from eppy import walk_hvac

e1 = [
    (("p_loop Supply Inlet", "epnode"), "Central_Chiller"),
    ("Central_Chiller", ("Central_Chiller_np1_node", "epnode")),
    (("Central_Chiller_np1_node", "epnode"), "np1"),
    ("np1", ("np1_np2_node", "epnode")),
    (("np1_np2_node", "epnode"), "np2"),
    ("np2", ("np2_Outlet_Node_Name", "epnode")),
    (("sb1_pipe_inlet", "epnode"), "sb1_pipe"),
    ("sb1_pipe", ("sb1_pipe_outlet", "epnode")),
    (("sb2_pipe_inlet", "epnode"), "sb2_pipe"),
    ("sb2_pipe", ("sb2_pipe_outlet", "epnode")),
    (("sb3_pipe_inlet", "epnode"), "sb3_pipe"),
    ("sb3_pipe", ("sb3_pipe_outlet", "epnode")),
    (("sb4_pipe_inlet", "epnode"), "sb4_pipe"),
    ("sb4_pipe", ("p_loop Supply Outlet", "epnode")),
    (("p_loop Demand Inlet", "epnode"), "db0_pipe"),
    ("db0_pipe", ("db0_pipe_outlet", "epnode")),
    (("db1_pipe_inlet", "epnode"), "db1_pipe"),
    ("db1_pipe", ("db1_pipe_outlet", "epnode")),
    (("db2_pipe_inlet", "epnode"), "db2_pipe"),
    ("db2_pipe", ("db2_pipe_outlet", "epnode")),
    (("db3_pipe_inlet", "epnode"), "db3_pipe"),
    ("db3_pipe", ("db3_pipe_outlet", "epnode")),
    (("db4_pipe_inlet", "epnode"), "db4_pipe"),
    ("db4_pipe", ("p_loop Demand Outlet", "epnode")),
    (("np2_Outlet_Node_Name", "epnode"), "p_loop_supply_splitter"),
    ("p_loop_supply_splitter", ("sb1_pipe_inlet", "epnode")),
    ("p_loop_supply_splitter", ("sb2_pipe_inlet", "epnode")),
    ("p_loop_supply_splitter", ("sb3_pipe_inlet", "epnode")),
    (("db0_pipe_outlet", "epnode"), "p_loop_demand_splitter"),
    ("p_loop_demand_splitter", ("db1_pipe_inlet", "epnode")),
    ("p_loop_demand_splitter", ("db2_pipe_inlet", "epnode")),
    ("p_loop_demand_splitter", ("db3_pipe_inlet", "epnode")),
    ("p_loop_supply_mixer", ("sb4_pipe_inlet", "epnode")),
    (("sb1_pipe_outlet", "epnode"), "p_loop_supply_mixer"),
    (("sb2_pipe_outlet", "epnode"), "p_loop_supply_mixer"),
    (("sb3_pipe_outlet", "epnode"), "p_loop_supply_mixer"),
    ("p_loop_demand_mixer", ("db4_pipe_inlet", "epnode")),
    (("db1_pipe_outlet", "epnode"), "p_loop_demand_mixer"),
    (("db2_pipe_outlet", "epnode"), "p_loop_demand_mixer"),
    (("db3_pipe_outlet", "epnode"), "p_loop_demand_mixer"),
]

e2 = (
    (("VAV Sys 1 Inlet Node", "epnode"), "Return Fan 1"),
    ("Return Fan 1", ("Return Fan 1 Outlet Node", "epnode")),
    (("Return Fan 1 Outlet Node", "epnode"), "OA Sys 1"),
    ("OA Sys 1", ("Mixed Air Node 1", "epnode")),
    (("Mixed Air Node 1", "epnode"), "Main Cooling Coil 1"),
    ("Main Cooling Coil 1", ("Main Cooling Coil 1 Outlet Node", "epnode")),
    (("Main Cooling Coil 1 Outlet Node", "epnode"), "Main Heating Coil 1"),
    ("Main Heating Coil 1", ("Main Heating Coil 1 Outlet Node", "epnode")),
    (("Main Heating Coil 1 Outlet Node", "epnode"), "Supply Fan 1"),
    ("Supply Fan 1", ("VAV Sys 1 Outlet Node", "epnode")),
    (("HW Supply Inlet Node", "epnode"), "HW Circ Pump"),
    ("HW Circ Pump", ("HW Pump Outlet Node", "epnode")),
    (("Central Boiler Inlet Node", "epnode"), "Central Boiler"),
    ("Central Boiler", ("Central Boiler Outlet Node", "epnode")),
    (("Heating Supply Bypass Inlet Node", "epnode"), "Heating Supply Side Bypass"),
    ("Heating Supply Side Bypass", ("Heating Supply Bypass Outlet Node", "epnode")),
    (("Heating Supply Exit Pipe Inlet Node", "epnode"), "Heating Supply Outlet"),
    ("Heating Supply Outlet", ("HW Supply Outlet Node", "epnode")),
    (("HW Demand Inlet Node", "epnode"), "Heating Demand Inlet Pipe"),
    ("Heating Demand Inlet Pipe", ("HW Demand Entrance Pipe Outlet Node", "epnode")),
    (("HW Demand Exit Pipe Inlet Node", "epnode"), "Heating Demand Outlet Pipe"),
    ("Heating Demand Outlet Pipe", ("HW Demand Outlet Node", "epnode")),
    (("SPACE1-1 Zone Coil Water In Node", "epnode"), "SPACE1-1 Zone Coil"),
    ("SPACE1-1 Zone Coil", ("SPACE1-1 Zone Coil Water Out Node", "epnode")),
    (("SPACE2-1 Zone Coil Water In Node", "epnode"), "SPACE2-1 Zone Coil"),
    ("SPACE2-1 Zone Coil", ("SPACE2-1 Zone Coil Water Out Node", "epnode")),
    (("SPACE3-1 Zone Coil Water In Node", "epnode"), "SPACE3-1 Zone Coil"),
    ("SPACE3-1 Zone Coil", ("SPACE3-1 Zone Coil Water Out Node", "epnode")),
    (("SPACE4-1 Zone Coil Water In Node", "epnode"), "SPACE4-1 Zone Coil"),
    ("SPACE4-1 Zone Coil", ("SPACE4-1 Zone Coil Water Out Node", "epnode")),
    (("SPACE5-1 Zone Coil Water In Node", "epnode"), "SPACE5-1 Zone Coil"),
    ("SPACE5-1 Zone Coil", ("SPACE5-1 Zone Coil Water Out Node", "epnode")),
    (("Main Heating Coil 1 Water Inlet Node", "epnode"), "Main Heating Coil 1"),
    ("Main Heating Coil 1", ("Main Heating Coil 1 Water Outlet Node", "epnode")),
    (("Heating Demand Bypass Inlet Node", "epnode"), "Heating Demand Bypass"),
    ("Heating Demand Bypass", ("Heating Demand Bypass Outlet Node", "epnode")),
    (("CW Demand Inlet Node", "epnode"), "Cooling Demand Side Inlet Pipe"),
    (
        "Cooling Demand Side Inlet Pipe",
        ("CW Demand Entrance Pipe Outlet Node", "epnode"),
    ),
    (("Main Cooling Coil 1 Water Inlet Node", "epnode"), "Main Cooling Coil 1"),
    ("Main Cooling Coil 1", ("Main Cooling Coil 1 Water Outlet Node", "epnode")),
    (("CW Demand Bypass Inlet Node", "epnode"), "Cooling Demand Side Bypass"),
    ("Cooling Demand Side Bypass", ("CW Demand Bypass Outlet Node", "epnode")),
    (("CW Demand Exit Pipe Inlet Node", "epnode"), "CW Demand Side Outlet Pipe"),
    ("CW Demand Side Outlet Pipe", ("CW Demand Outlet Node", "epnode")),
    (("Supply Side Exit Pipe Inlet Node", "epnode"), "Supply Side Outlet Pipe"),
    ("Supply Side Outlet Pipe", ("CW Supply Outlet Node", "epnode")),
    (("CW Supply Inlet Node", "epnode"), "CW Circ Pump"),
    ("CW Circ Pump", ("CW Pump Outlet Node", "epnode")),
    (("Central Chiller Inlet Node", "epnode"), "Central Chiller"),
    ("Central Chiller", ("Central Chiller Outlet Node", "epnode")),
    (("CW Supply Bypass Inlet Node", "epnode"), "Supply Side Bypass"),
    ("Supply Side Bypass", ("CW Supply Bypass Outlet Node", "epnode")),
    (("Condenser Supply Inlet Node", "epnode"), "Cond Circ Pump"),
    ("Cond Circ Pump", ("Condenser Pump Outlet Node", "epnode")),
    (("Condenser Tower Inlet Node", "epnode"), "Central Tower"),
    ("Central Tower", ("Condenser Tower Outlet Node", "epnode")),
    (("Cond Supply Bypass Inlet Node", "epnode"), "Condenser Supply Side Bypass"),
    ("Condenser Supply Side Bypass", ("Cond Supply Bypass Outlet Node", "epnode")),
    (("Condenser Supply Exit Pipe Inlet Node", "epnode"), "Condenser Supply Outlet"),
    ("Condenser Supply Outlet", ("Condenser Supply Outlet Node", "epnode")),
    (("Condenser Demand Inlet Node", "epnode"), "Condenser Demand Inlet Pipe"),
    (
        "Condenser Demand Inlet Pipe",
        ("Condenser Demand Entrance Pipe Outlet Node", "epnode"),
    ),
    (("Central Chiller Condenser Inlet Node", "epnode"), "Central Chiller"),
    ("Central Chiller", ("Central Chiller Condenser Outlet Node", "epnode")),
    (("Cond Demand Bypass Inlet Node", "epnode"), "Condenser Demand Side Bypass"),
    ("Condenser Demand Side Bypass", ("Cond Demand Bypass Outlet Node", "epnode")),
    (
        ("Condenser Demand Exit Pipe Inlet Node", "epnode"),
        "Condenser Demand Outlet Pipe",
    ),
    ("Condenser Demand Outlet Pipe", ("Condenser Demand Outlet Node", "epnode")),
    (("HW Demand Entrance Pipe Outlet Node", "epnode"), "Heating Demand Splitter"),
    ("Heating Demand Splitter", ("SPACE1-1 Zone Coil Water In Node", "epnode")),
    ("Heating Demand Splitter", ("SPACE2-1 Zone Coil Water In Node", "epnode")),
    ("Heating Demand Splitter", ("SPACE3-1 Zone Coil Water In Node", "epnode")),
    ("Heating Demand Splitter", ("SPACE4-1 Zone Coil Water In Node", "epnode")),
    ("Heating Demand Splitter", ("SPACE5-1 Zone Coil Water In Node", "epnode")),
    ("Heating Demand Splitter", ("Main Heating Coil 1 Water Inlet Node", "epnode")),
    ("Heating Demand Splitter", ("Heating Demand Bypass Inlet Node", "epnode")),
    (("HW Pump Outlet Node", "epnode"), "Heating Supply Splitter"),
    ("Heating Supply Splitter", ("Central Boiler Inlet Node", "epnode")),
    ("Heating Supply Splitter", ("Heating Supply Bypass Inlet Node", "epnode")),
    (("CW Pump Outlet Node", "epnode"), "CW Loop Splitter"),
    ("CW Loop Splitter", ("Central Chiller Inlet Node", "epnode")),
    ("CW Loop Splitter", ("CW Supply Bypass Inlet Node", "epnode")),
    (("CW Demand Entrance Pipe Outlet Node", "epnode"), "CW Demand Splitter"),
    ("CW Demand Splitter", ("Main Cooling Coil 1 Water Inlet Node", "epnode")),
    ("CW Demand Splitter", ("CW Demand Bypass Inlet Node", "epnode")),
    (
        ("Condenser Demand Entrance Pipe Outlet Node", "epnode"),
        "Condenser Demand Splitter",
    ),
    ("Condenser Demand Splitter", ("Central Chiller Condenser Inlet Node", "epnode")),
    ("Condenser Demand Splitter", ("Cond Demand Bypass Inlet Node", "epnode")),
    (("Condenser Pump Outlet Node", "epnode"), "Condenser Supply Splitter"),
    ("Condenser Supply Splitter", ("Condenser Tower Inlet Node", "epnode")),
    ("Condenser Supply Splitter", ("Cond Supply Bypass Inlet Node", "epnode")),
    ("Heating Demand Mixer", ("HW Demand Exit Pipe Inlet Node", "epnode")),
    (("SPACE1-1 Zone Coil Water Out Node", "epnode"), "Heating Demand Mixer"),
    (("SPACE2-1 Zone Coil Water Out Node", "epnode"), "Heating Demand Mixer"),
    (("SPACE3-1 Zone Coil Water Out Node", "epnode"), "Heating Demand Mixer"),
    (("SPACE4-1 Zone Coil Water Out Node", "epnode"), "Heating Demand Mixer"),
    (("SPACE5-1 Zone Coil Water Out Node", "epnode"), "Heating Demand Mixer"),
    (("Main Heating Coil 1 Water Outlet Node", "epnode"), "Heating Demand Mixer"),
    (("Heating Demand Bypass Outlet Node", "epnode"), "Heating Demand Mixer"),
    ("Heating Supply Mixer", ("Heating Supply Exit Pipe Inlet Node", "epnode")),
    (("Central Boiler Outlet Node", "epnode"), "Heating Supply Mixer"),
    (("Heating Supply Bypass Outlet Node", "epnode"), "Heating Supply Mixer"),
    ("CW Loop Mixer", ("Supply Side Exit Pipe Inlet Node", "epnode")),
    (("Central Chiller Outlet Node", "epnode"), "CW Loop Mixer"),
    (("CW Supply Bypass Outlet Node", "epnode"), "CW Loop Mixer"),
    ("CW Demand Mixer", ("CW Demand Exit Pipe Inlet Node", "epnode")),
    (("Main Cooling Coil 1 Water Outlet Node", "epnode"), "CW Demand Mixer"),
    (("CW Demand Bypass Outlet Node", "epnode"), "CW Demand Mixer"),
    ("Condenser Demand Mixer", ("Condenser Demand Exit Pipe Inlet Node", "epnode")),
    (("Central Chiller Condenser Outlet Node", "epnode"), "Condenser Demand Mixer"),
    (("Cond Demand Bypass Outlet Node", "epnode"), "Condenser Demand Mixer"),
    ("Condenser Supply Mixer", ("Condenser Supply Exit Pipe Inlet Node", "epnode")),
    (("Condenser Tower Outlet Node", "epnode"), "Condenser Supply Mixer"),
    (("Cond Supply Bypass Outlet Node", "epnode"), "Condenser Supply Mixer"),
    (("Zone Eq In Node", "epnode"), "Zone Supply Air Splitter 1"),
    ("Zone Supply Air Splitter 1", ("SPACE1-1 ATU In Node", "epnode")),
    ("Zone Supply Air Splitter 1", ("SPACE2-1 ATU In Node", "epnode")),
    ("Zone Supply Air Splitter 1", ("SPACE3-1 ATU In Node", "epnode")),
    ("Zone Supply Air Splitter 1", ("SPACE4-1 ATU In Node", "epnode")),
    ("Zone Supply Air Splitter 1", ("SPACE5-1 ATU In Node", "epnode")),
    ("Return-Plenum-1", ("PLENUM-1 Out Node", "epnode")),
    (("SPACE1-1 Out Node", "epnode"), "Return-Plenum-1"),
    (("SPACE2-1 Out Node", "epnode"), "Return-Plenum-1"),
    (("SPACE3-1 Out Node", "epnode"), "Return-Plenum-1"),
    (("SPACE4-1 Out Node", "epnode"), "Return-Plenum-1"),
    (("SPACE5-1 Out Node", "epnode"), "Return-Plenum-1"),
    ("SPACE1-1", ("SPACE1-1 Out Node", "epnode")),
    ("SPACE2-1", ("SPACE2-1 Out Node", "epnode")),
    ("SPACE3-1", ("SPACE3-1 Out Node", "epnode")),
    ("SPACE4-1", ("SPACE4-1 Out Node", "epnode")),
    ("SPACE5-1", ("SPACE5-1 Out Node", "epnode")),
    ("SPACE1-1 ATU", "SPACE1-1"),
    ("SPACE2-1 ATU", "SPACE2-1"),
    ("SPACE3-1 ATU", "SPACE3-1"),
    ("SPACE4-1 ATU", "SPACE4-1"),
    ("SPACE5-1 ATU", "SPACE5-1"),
    ("SPACE1-1 VAV Reheat", "SPACE1-1 ATU"),
    ("SPACE2-1 VAV Reheat", "SPACE2-1 ATU"),
    ("SPACE3-1 VAV Reheat", "SPACE3-1 ATU"),
    ("SPACE4-1 VAV Reheat", "SPACE4-1 ATU"),
    ("SPACE5-1 VAV Reheat", "SPACE5-1 ATU"),
    (("SPACE1-1 ATU In Node", "epnode"), "SPACE1-1 VAV Reheat"),
    (("SPACE2-1 ATU In Node", "epnode"), "SPACE2-1 VAV Reheat"),
    (("SPACE3-1 ATU In Node", "epnode"), "SPACE3-1 VAV Reheat"),
    (("SPACE4-1 ATU In Node", "epnode"), "SPACE4-1 VAV Reheat"),
    (("SPACE5-1 ATU In Node", "epnode"), "SPACE5-1 VAV Reheat"),
)


def test_nextnode():
    """py.test for nextnode"""
    edges = e1
    nexts = [
        "Central_Chiller",
        "np1",
        "np2",
        "p_loop_supply_splitter",
        "sb1_pipe",
        "p_loop_supply_mixer",
        "sb4_pipe",
    ]

    c = "Central_Chiller"

    n = 0
    i = 0  # counter
    while n == 0:  # endless loop
        assert nexts[i] == c
        i += 1
        nextcs = walk_hvac.nextnode(edges, c)
        if len(nextcs) == 0:
            break
        c = nextcs[0]
    # test links that have no nodes - equipment in spaces
    edges = e2
    connections = (
        ("SPACE2-1 ATU", "SPACE2-1"),
        ("SPACE3-1 ATU", "SPACE3-1"),
        ("SPACE4-1 ATU", "SPACE4-1"),
        ("SPACE5-1 ATU", "SPACE5-1"),
        ("SPACE1-1 VAV Reheat", "SPACE1-1 ATU"),
        ("SPACE2-1 VAV Reheat", "SPACE2-1 ATU"),
        ("SPACE3-1 VAV Reheat", "SPACE3-1 ATU"),
        ("SPACE4-1 VAV Reheat", "SPACE4-1 ATU"),
        ("SPACE5-1 VAV Reheat", "SPACE5-1 ATU"),
    )
    for comp, nextcomp in connections:
        result = walk_hvac.nextnode(edges, comp)
        assert result == [nextcomp]


def test_prevnode():
    """py.test for prevnode"""
    edges = e1
    prevs = [
        "sb4_pipe",
        "p_loop_supply_mixer",
        "sb1_pipe",
        "p_loop_supply_splitter",
        "np2",
        "np1",
        "Central_Chiller",
    ]

    c = "sb4_pipe"
    i = 0
    n = 0
    while n == 0:
        assert prevs[i] == c
        i += 1
        prevcs = walk_hvac.prevnode(edges, c)
        if len(prevcs) == 0:
            break
        c = prevcs[0]
    # test links that have no nodes - equipment in spaces
    edges = e2
    connections = (
        ("SPACE2-1 ATU", "SPACE2-1"),
        ("SPACE3-1 ATU", "SPACE3-1"),
        ("SPACE4-1 ATU", "SPACE4-1"),
        ("SPACE5-1 ATU", "SPACE5-1"),
        ("SPACE1-1 VAV Reheat", "SPACE1-1 ATU"),
        ("SPACE2-1 VAV Reheat", "SPACE2-1 ATU"),
        ("SPACE3-1 VAV Reheat", "SPACE3-1 ATU"),
        ("SPACE4-1 VAV Reheat", "SPACE4-1 ATU"),
        ("SPACE5-1 VAV Reheat", "SPACE5-1 ATU"),
    )
    for prevcomp, comp in connections:
        result = walk_hvac.prevnode(edges, comp)
        assert result == [prevcomp]



import glob
import os

from eppy.iddcurrent import iddcurrent
from eppy.modeleditor import IDF
from eppy.pytest_helpers import IDD_FILES
from io import StringIO

import eppy.snippet as snippet


iddsnippet = iddcurrent.iddtxt
idfsnippet = snippet.idfsnippet

iddfhandle = StringIO(iddcurrent.iddtxt)
IDF.setiddname(iddfhandle)

idd = os.path.join(IDD_FILES, "Energy+V8_1_0.idd")
OUTPUT_DIR = "C:\EnergyPlusV8-5-0\ExampleFiles\loopdiagrams"

idfs = glob.glob(OUTPUT_DIR + "\*.idf")
dots = glob.glob(OUTPUT_DIR + "\*.dot")
for idf in idfs:
    os.remove(idf)
for dot in dots:
    os.remove(dot)



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""py.test for class IDF"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from io import StringIO

from eppy.iddcurrent import iddcurrent
from eppy.modeleditor import IDF


def test_IDF():
    """py.test for class IDF"""
    stored_idd = IDF.iddname
    IDF.iddname = None
    assert IDF.iddname == None
    IDF.setiddname("gumby", testing=True)
    assert IDF.iddname == "gumby"
    IDF.setiddname("karamba", testing=True)
    assert IDF.iddname != "karamba"
    assert IDF.iddname == "gumby"
    IDF.iddname = stored_idd


iddsnippet = iddcurrent.iddtxt
iddfhandle = StringIO(iddsnippet)


if IDF.getiddname() == None:
    IDF.setiddname(iddfhandle)


class TestIDF(object):

    """py.test for IDF function"""

    def test_removeidfobject(self):
        """py.test for IDF.removeidfobject """
        idftxt = ""
        idfhandle = StringIO(idftxt)
        idf = IDF(idfhandle)
        key = "BUILDING"
        idf.newidfobject(key, Name="Building_remove")
        idf.newidfobject(key, Name="Building1")
        idf.newidfobject(key, Name="Building_remove")
        idf.newidfobject(key, Name="Building2")
        buildings = idf.idfobjects["building"]
        removethis = buildings[-2]
        idf.removeidfobject(removethis)
        assert buildings[2].Name == "Building2"
        assert idf.model.dt[key][2][1] == "Building2"

    def test_popidfobject(self):
        idftxt = ""
        idfhandle = StringIO(idftxt)
        idf = IDF(idfhandle)
        key = "BUILDING"
        idf.newidfobject(key, Name="Building_remove")
        idf.newidfobject(key, Name="Building1")
        idf.newidfobject(key, Name="Building_remove")
        idf.newidfobject(key, Name="Building2")
        buildings = idf.idfobjects["building"]
        removethis = buildings[-2]
        idf.popidfobject(key, 2)
        assert buildings[2].Name == "Building2"
        assert idf.model.dt[key][2][1] == "Building2"



# Copyright (c) 2020 Cheng Cui
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""py.test for bunch_subclass_functions"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from io import StringIO

from eppy.iddcurrent import iddcurrent
from eppy.modeleditor import IDF
from eppy.pytest_helpers import almostequal


iddtxt = iddcurrent.iddtxt
iddfhandle = StringIO(iddcurrent.iddtxt)
if IDF.getiddname() == None:
    IDF.setiddname(iddfhandle)

idftxt = """Version,8.0;

    Building,
        building,                !- Name
        45;                      !- North Axis {deg} 

    Zone,
        West Zone,               !- Name
        30,                      !- Direction of Relative North {deg}
        0, 0, 0;                            !- X,Y,Z  {m}

    GlobalGeometryRules,
        UpperLeftCorner,         !- Starting Vertex Position
        CounterClockWise,        !- Vertex Entry Direction
        Relative;                !- Coordinate System

    BuildingSurface:Detailed,
        Zn001:Wall001,           !- Name
        Wall,                    !- Surface Type
        EXTWALL80,               !- Construction Name
        West Zone,               !- Zone Name
        Outdoors,                !- Outside Boundary Condition
        ,                        !- Outside Boundary Condition Object
        SunExposed,              !- Sun Exposure
        WindExposed,             !- Wind Exposure
        0.5000000,               !- View Factor to Ground
        4,                       !- Number of Vertices
        0,0,3.048000,  !- X,Y,Z ==> Vertex 1 {m}
        0,0,0,  !- X,Y,Z ==> Vertex 2 {m}
        6.096000,0,0,  !- X,Y,Z ==> Vertex 3 {m}
        6.096000,0,3.048000;  !- X,Y,Z ==> Vertex 4 {m}

"""


def test_surface_function():
    fhandle = StringIO(idftxt)
    idf = IDF(fhandle)
    surface = idf.idfobjects["BuildingSurface:Detailed"][0]

    # test the azimuth
    assert almostequal(surface.azimuth, 180, places=3) == True

    # test the true azimuth
    assert almostequal(surface.true_azimuth, 255, places=3) == True



import os
import shutil
import sys
from importlib import reload

from six import StringIO

from eppy import modeleditor
from eppy.runner.run_functions import parse_error, EnergyPlusRunError


def test_capture_stderr():
    tmp_out = StringIO()
    sys.stderr = tmp_out
    sys.stderr.write("I am in stderr")
    msg = parse_error(tmp_out, "C:/notafile")
    assert "<File not found>" in msg
    assert "I am in stderr" in msg
    sys.stderr = sys.__stderr__


def test_capture_real_error(test_idf):
    test_idf.newidfobject(
        "HVACTemplate:Thermostat",
        Name="thermostat VRF",
        Heating_Setpoint_Schedule_Name=15,
        Constant_Cooling_Setpoint=25,
    )
    rundir = "test_capture_real_error"
    os.mkdir(rundir)
    try:
        test_idf.run(output_directory=rundir)
    except EnergyPlusRunError as e:
        assert "invalid Heating Setpoint Temperature Schedule" in str(e)
    finally:
        shutil.rmtree(rundir)
        reload(modeleditor)



from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import io
import codecs
import os
import sys

here = os.path.abspath(os.path.dirname(__file__))


def read(*filenames, **kwargs):
    encoding = kwargs.get("encoding", "utf-8")
    sep = kwargs.get("sep", "\n")
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


long_description = read("README.rst")


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        errcode = pytest.main(self.test_args)
        sys.exit(errcode)


setup(
    name="eppy",
    version="0.5.57",
    url="https://github.com/santoshphilip/eppy",
    license="MIT License",
    author="Santosh Philip",
    tests_require=["pytest"],
    cmdclass={"test": PyTest},
    author_email="eppy_scripting@yahoo.com",
    description="Scripting language for E+ idf files, and E+ output files",
    long_description=long_description,  # TODO set this up
    packages=[
        "eppy",
        "eppy.EPlusInterfaceFunctions",
        "eppy.geometry",
        "eppy.constructions",
        "eppy.runner",
        "eppy.results",
        "eppy.useful_scripts",
    ],
    include_package_data=True,
    platforms="any",
    test_suite="eppy.test.test_eppy",  # TODO make test_eppy
    install_requires=[
        "munch>=2.0.2",
        "beautifulsoup4<=4.8.0",
        "tinynumpy>=1.2.1",
        "six>=1.10.0",
        "decorator>=4.0.10",
        "lxml>=3.8.0",
        "future",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Development Status :: 4 - Beta",
        "Natural Language :: English",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
    ],
    extras_require={
        ':python_version>="3.0"': ["pydot3k", "pyparsing>=2.4.7"],
        "testing": ["pytest"],
    },
)
